/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.applinks

import android.content.Context
import android.net.Uri
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.asynctask.AfterActionWithType
import com.dropbox.android.asynctask.DbxAsyncTask
import com.dropbox.android.filemanager.ApiManager
import com.dropbox.core.legacy_api.exception.DropboxException

/**
 * Given a uri, that might redirect, returns the first uri in the redirect change. I.e. if a.com
 * redirects to b.com which redirects to c.com then b.com will be returned.
 *
 * If the given uri does not redirect, the given uri will be returned.
 */
class FetchRedirectUriAsyncTask<T> (
    context: Context,
    private val apiManager: ApiManager,
    private val uri: String
) : DbxAsyncTask<Void, AfterActionWithType<T>> (context)
        where T : FetchRedirectUriAsyncTask.FetchRedirectUriCallbacks,
              T : BaseActivity {

    /**
     * Interface used for [FetchRedirectUriAsyncTask] callbacks.
     */
    interface FetchRedirectUriCallbacks {
        /**
         * Called when successfully fetched the redirect uri.
         */
        fun onRedirectFetched(redirectUri: Uri)

        /**
         * Called when failed to fetch redirect uri.
         */
        fun onRedirectFetchFailed(exception: DropboxException)
    }

    private inner class SuccessAction(val redirectUri: Uri) : AfterActionWithType<T> {
        override fun doAfterTaskCompletion(context: T) {
            context.onRedirectFetched(redirectUri)
        }
    }

    private inner class FailureAction(val exception: DropboxException) : AfterActionWithType<T> {
        override fun doAfterTaskCompletion(context: T) {
            context.onRedirectFetchFailed(exception)
        }
    }

    public override fun doInBackground(): AfterActionWithType<T>? {
        try {
            return SuccessAction(apiManager.getRedirectUri(uri))
        } catch (e: DropboxException) {
            return FailureAction(e)
        }
    }

    // Our constructor requires that the Context is a "T" type, so the cast here is safe
    @Suppress("unchecked_cast")
    public override fun after(context: Context, result: AfterActionWithType<T>?) {
        result?.doAfterTaskCompletion(context as T)
    }
}
