/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
// dbx-min-coverage: 69% because the rest of this async task is not testable
package com.dropbox.android.asynctask

import android.content.Context
import com.dropbox.android.activity.DropboxChooserResultListener
import com.dropbox.android.filemanager.DbxChooserResult
import com.dropbox.android.filemanager.downloading.DownloadManager
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.PathHelper
import com.dropbox.core.DbxException
import com.dropbox.core.NetworkIOException
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.legacy_api.exception.DropboxIOException
import com.dropbox.core.legacy_api.exception.DropboxServerException
import com.dropbox.core.v2.DbxClientV2
import com.dropbox.core.v2.sharing.DirectLinkMetadata
import com.dropbox.hairball.entry.DropboxLocalEntry
import com.dropbox.hairball.taskqueue.TaskResult
import com.dropbox.internalclient.UserApi
import com.dropbox.product.dbapp.path.DropboxPath
import java.util.concurrent.atomic.AtomicReference

/**
 * For [ResultType.CHOOSER_FILE], fetches thumbnail links prior to running
 * the standard DownloadFileAsyncTaskBase logic, then delivers the result via
 * [DropboxChooserResultListener.onGetChooserFileMetadataSuccess].
 */
class ChooserDownloadAsyncTask<T>(
    context: T,
    val user: DbxUser,
    val entry: DropboxLocalEntry,
    private val appKey: String,
    private val useApiV2: Boolean
)
    : DownloadFileAsyncTaskBase<T, DropboxPath>(context, entry, PathHelper.create(entry.path, user))
        where T : Context, T : DropboxChooserResultListener {

    private val path: DropboxPath = entry.path
    private val userId: String = user.id
    private val apiV1: UserApi = user.api
    private val apiV2: DbxClientV2 = user.apiV2

    public override fun doInBackground(): AfterActionWithType<T> =
            if (useApiV2) {
                fetchDownloadLink()
            } else {
                fetchDownloadLinkLegacy()
            }

    private fun fetchDownloadLinkLegacy(): AfterActionWithType<T> =
        try {
            val links = apiV1.chooserShare(
                    listOf(path), appKey, "direct")
            // Now we download the file.
            val action = super.doInBackground()
            // If the super class returned our success action, set the link on it
            if (action is SuccessAction) {
                action.setLinkLegacy(links[0])
            }
            action as AfterActionWithType<T>
        } catch (e: DropboxException) {
            if (e is DropboxIOException) {
                createFailureAction(TaskResult.ResultType.NETWORK_ERROR)
            } else if (e is DropboxServerException) {
                createFailureAction(TaskResult.ResultType.TEMP_SERVER_ERROR)
            }
            createFailureAction(TaskResult.ResultType.FAILURE)
        }

    private fun fetchDownloadLink(): AfterActionWithType<T> =
        try {
            val link = apiV2
                    .sharing()
                    .createDirectLink(path.asCanonicalPath(), appKey)
            // Now we download the file.
            val action = super.doInBackground()
            // If the super class returned our success action, set the link on it
            if (action is SuccessAction) {
                action.setLink(link)
            }
            action as AfterActionWithType<T>
        } catch (e: NetworkIOException) {
            createFailureAction(TaskResult.ResultType.NETWORK_ERROR)
        } catch (e: DbxException) {
            createFailureAction(TaskResult.ResultType.FAILURE)
        }

    public override fun createSuccessAction(
        downloadResult: DownloadManager.Result<DropboxPath>
    ): AfterActionWithType<T> = SuccessAction(downloadResult)

    /**
     * Custom [AfterAction] that calls a callback on [DropboxChooserResultListener]
     */
    private inner class SuccessAction(
        private val downloadResult: DownloadManager.Result<DropboxPath>
    )
        : AfterActionWithType<T> {

        private val link = AtomicReference<DirectLinkMetadata?>()
        private val linkLegacy = AtomicReference<UserApi.ChooserLink?>()

        /**
         * Set the link part of the result after construction
         * @param newLink
         */
        fun setLink(newLink: DirectLinkMetadata) = link.set(newLink)

        /**
         * Set the link part of the legacy result after construction
         * @param newLink
         */
        fun setLinkLegacy(newLink: UserApi.ChooserLink) = linkLegacy.set(newLink)

        override fun doAfterTaskCompletion(context: T) {
            // Check link for null, since doInBackground may not have set it
            val currLink = link.get()
            val currLinkLegacy = linkLegacy.get()

            if (currLink != null) {
                context.onGetChooserFileSuccess(
                        downloadResult.localFilePath, userId, context,
                        DbxChooserResult.constructFromDirectLinkMetadata(currLink))
            } else if (currLinkLegacy != null) {
                context.onGetChooserFileSuccess(
                        downloadResult.localFilePath, userId, context,
                        DbxChooserResult.constructFromChooserLink(currLinkLegacy))
            }
        }
    }
}
