/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
// dbx-min-coverage: 62% because the rest of this async task is not testable
package com.dropbox.android.asynctask

import android.content.Context
import android.support.v4.app.FragmentActivity
import com.dropbox.android.R
import com.dropbox.android.activity.DropboxChooserResultListener
import com.dropbox.android.activity.dialog.TextProgressDialogFrag
import com.dropbox.android.filemanager.DbxChooserResult
import com.dropbox.android.filemanager.GetLinkResult
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.ToastFactory
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.core.DbxException
import com.dropbox.core.NetworkIOException
import com.dropbox.core.legacy_api.exception.DropboxIOException
import com.dropbox.product.dbapp.path.DropboxPath

/**
 * For [ResultType.CHOOSER_DIRECT_LINK], fetches the metadata for selected direct link
 */
private const val TAG = "GetDirectLinkAsyncTask"
class GetDirectLinkAsyncTask(
    context: Context,
    private val user: DbxUser,
    private val path: DropboxPath,
    private val appKey: String,
    private val useApiV2: Boolean
)
    : DbxAsyncTask<Void, GetLinkResult>(context) {

    override fun before(context: Context) {
        check(context is DropboxChooserResultListener) {
            "Class should implement DropboxChooserResultListener" }
        disableDialog()
        val frag = TextProgressDialogFrag.newInstance(R.string.sharing_dialog_message)
        frag.show(context, (context as FragmentActivity).supportFragmentManager)
    }

    public override fun doInBackground(): GetLinkResult =
            if (useApiV2) {
                getDirectLink()
            } else {
                getDirectLinkLegacy()
            }

    override fun after(context: Context, result: GetLinkResult?) {
        TextProgressDialogFrag.dismiss(
                (context as FragmentActivity).supportFragmentManager)

        when (result?.status) {
            GetLinkResult.Status.SUCCESS -> {
                if (context is DropboxChooserResultListener) {
                    context.onGetDirectLinkSuccess(result.chooserResult)
                }
            }
            GetLinkResult.Status.FAILED_NETWORK_ERROR, GetLinkResult.Status.FAILED_UNKNOWN -> {
                ToastFactory.showText(context, result.userError!!)
            }
        }
    }

    private fun getDirectLink(): GetLinkResult =
        // create_direct_link: Calling this endpoint on a path that already has a valid direct link returns the same direct
        // link, but with the expiration reset as if the direct link were created for the first time.
        try {
            val link = user.apiV2
                    .sharing()
                    .createDirectLink(path.asCanonicalPath(), appKey)
            GetLinkResult.forSuccess(DbxChooserResult.constructFromDirectLinkMetadata(link))
        } catch (e: NetworkIOException) {
            DbxLog.external(TAG, "Error in GetDirectLinkAsyncTask", e)
            GetLinkResult.forFailure(
                    GetLinkResult.Status.FAILED_NETWORK_ERROR, R.string.error_network_error)
        } catch (e: DbxException) {
            DbxLog.external(TAG, "Error in GetDirectLinkAsyncTask", e)
            GetLinkResult.forFailure(
                    GetLinkResult.Status.FAILED_UNKNOWN, R.string.error_generic)
        }

    private fun getDirectLinkLegacy(): GetLinkResult =
            try {
                val links = user.api.chooserShare(listOf(path), appKey, "direct")
                GetLinkResult.forSuccess(DbxChooserResult.constructFromChooserLink(links[0]))
            } catch (e: DropboxIOException) {
                DbxLog.external(TAG, "Error in GetLinkAsyncTask", e)
                GetLinkResult.forFailure(
                    GetLinkResult.Status.FAILED_NETWORK_ERROR, R.string.error_network_error)
            } catch (e: Exception) {
                DbxLog.external(TAG, "Error in GetLinkAsyncTask", e)
                GetLinkResult.forFailure(
                        GetLinkResult.Status.FAILED_UNKNOWN, R.string.error_generic)
            }
}
