/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
// dbx-min-coverage: 68% because unable to the rest of this async task is not testable
package com.dropbox.android.asynctask

import android.content.Context
import android.support.v4.app.FragmentActivity
import com.dropbox.android.R
import com.dropbox.android.activity.DropboxChooserResultListener
import com.dropbox.android.activity.dialog.TextProgressDialogFrag
import com.dropbox.android.filemanager.DbxChooserResult
import com.dropbox.android.filemanager.GetLinkResult
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.ToastFactory
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.core.DbxException
import com.dropbox.core.NetworkIOException
import com.dropbox.core.legacy_api.exception.DropboxIOException
import com.dropbox.core.v2.sharing.ListSharedLinksResult
import com.dropbox.product.dbapp.path.DropboxPath

/**
 * For [ResultType.CHOOSER_PREVIEW_LINK], checks if there is an existing shared link. If not, create a
 * new one and return the metadata for it
 */
private const val TAG = "GetPreviewLinkAsyncTask"
class GetPreviewLinkAsyncTask(
    context: Context,
    private val user: DbxUser,
    private val path: DropboxPath,
    private val appKey: String,
    private val useApiV2: Boolean
)
    : DbxAsyncTask<Void, GetLinkResult>(context) {

    override fun before(context: Context) {
        check(context is DropboxChooserResultListener) {
            "Class should implement DropboxChooserResultListener" }
        disableDialog()
        val frag = TextProgressDialogFrag.newInstance(R.string.sharing_dialog_message)
        frag.show(context, (context as FragmentActivity).supportFragmentManager)
    }

    public override fun doInBackground(): GetLinkResult =
            if (useApiV2) {
                getSharedLink()
            } else {
                getSharedLinkLegacy()
            }

    private fun getSharedLinkLegacy(): GetLinkResult =
            try {
                val links = user.api.chooserShare(listOf(path), appKey, "preview")
                GetLinkResult.forSuccess(DbxChooserResult.constructFromChooserLink(links[0]))
            } catch (e: DropboxIOException) {
                DbxLog.external(TAG, "Error in GetLinkAsyncTask", e)
                GetLinkResult.forFailure(
                        GetLinkResult.Status.FAILED_NETWORK_ERROR, R.string.error_network_error)
            } catch (e: Exception) {
                DbxLog.external(TAG, "Error in GetLinkAsyncTask", e)
                GetLinkResult.forFailure(
                        GetLinkResult.Status.FAILED_UNKNOWN, R.string.error_generic)
            }

    private fun getSharedLink(): GetLinkResult {
        // First, see if there is an existing link here (attempting to create a link will fail if one already exists)
        val linksResult: ListSharedLinksResult
        try {
            linksResult = user.apiV2.sharing()
                    .listSharedLinksBuilder()
                    .withPath(path.asCanonicalPath())
                    .withAppKey(appKey)
                    .withDirectOnly(true)
                    .withCursor(null)
                    .start()
        } catch (e: NetworkIOException) {
            DbxLog.external(TAG, "Error in GetPreviewLinkAsyncTask", e)
            return GetLinkResult.forFailure(
                    GetLinkResult.Status.FAILED_NETWORK_ERROR, R.string.error_network_error)
        } catch (e: DbxException) {
            DbxLog.external(TAG, "Error in GetPreviewLinkAsyncTask", e)
            return GetLinkResult.forFailure(
                    GetLinkResult.Status.FAILED_UNKNOWN, R.string.error_generic)
        }

        return if (!linksResult.links.isEmpty()) {
            // if there is a link here, we're done
            GetLinkResult.forSuccess(
                    DbxChooserResult.constructFromSharedLinkMetadata(linksResult.links[0]))
        } else {
            try {
                // create a new link
                val sharedLink = user.apiV2.sharing()
                        .createSharedLinkWithSettingsBuilder(path.asCanonicalPath())
                        .withAppKey(appKey)
                        .withSettings(null)
                        .start()
                GetLinkResult.forSuccess(
                        DbxChooserResult.constructFromSharedLinkMetadata(sharedLink))
            } catch (e: NetworkIOException) {
                DbxLog.external(TAG, "Error in GetPreviewLinkAsyncTask", e)
                GetLinkResult.forFailure(
                        GetLinkResult.Status.FAILED_NETWORK_ERROR, R.string.error_network_error)
            } catch (e: DbxException) {
                DbxLog.external(TAG, "Error in GetPreviewLinkAsyncTask", e)
                GetLinkResult.forFailure(
                        GetLinkResult.Status.FAILED_UNKNOWN, R.string.error_generic)
            }
        }
    }

    override fun after(context: Context, result: GetLinkResult?) {
        TextProgressDialogFrag.dismiss(
                (context as FragmentActivity).supportFragmentManager)
        when (result?.status) {
            GetLinkResult.Status.SUCCESS -> {
                if (context is DropboxChooserResultListener) {
                    context.onGetPreviewLinkSuccess(result.chooserResult)
                }
            }
            GetLinkResult.Status.FAILED_NETWORK_ERROR, GetLinkResult.Status.FAILED_UNKNOWN -> {
                ToastFactory.showText(context, result.userError!!)
            }
        }
    }
}
