/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.camerauploads

import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.DbxUserManager
import com.dropbox.android.user.UserCapability
import com.dropbox.base.error.DbxException
import com.dropbox.core.stormcrow.StormcrowMobileAndroidCameraUploadAccountSwitching
import com.dropbox.core.stormcrow.StormcrowMobileAndroidShowCuEnableBannerInHome
import com.dropbox.core.stormcrow.StormcrowMobileAndroidShowCuUpsellBannerInHome
import com.dropbox.core.stormcrow.StormcrowMobileAndroidCameraUpload

internal class CameraUploadsGatingManagerImpl(
    private val userManager: DbxUserManager,
    stateChangeModel: CameraUploadsUserStateChangeModel
) : CameraUploadsGatingManager, CameraUploadsUserStateChangeModel.Listener {

    init {
        // TODO(ageiduschek): Once account switching for biz CU is entirely enabled, delete this -
        // it will no longer be referenced
        stateChangeModel.registerListener(this)
    }

    // TODO(ageiduschek): Once account switching for biz CU is entirely enabled, delete this -
    // it will no longer be referenced
    private var cuCandidateId: String? = null

    // TODO(ageiduschek): Deprecate all external references to this. This should be an
    // implementation detail of this class.
    override val cuUser: DbxUser?
        get() = userManager.users?.getUserById(cuCandidateId)?.takeIf {
            getUserCuAvailability(it) != UserCapability.Availability.NEVER_AVAILABLE
        }

    val cuEnabledUser: DbxUser?
        get() = userManager.users?.allUsers()?.filter(::isCUEnabledForUser)?.getOrNull(0)

    override fun onCUCandidateUserChanged(cuCandidateUser: DbxUser?) {
        cuCandidateId = cuCandidateUser?.id
    }

    override fun canEnableCuForUser(user: DbxUser?): Boolean =
            user != null &&
            mayEnableCuIfCapabilityIsAvailableForUser(user) &&
            getUserCuAvailability(user) == UserCapability.Availability.AVAILABLE

    override fun canEnableCuWithUpgradeForUser(user: DbxUser?): Boolean =
            user != null &&
            mayEnableCuIfCapabilityIsAvailableForUser(user) &&
            getUserCuAvailability(user) == UserCapability.Availability.CAN_BECOME_AVAILABLE

    override fun shouldShowCuUIInPhotosTabForUser(user: DbxUser?): Boolean {
        if (user == null) {
            return false
        }

        return user.let {
            val hasCuEnabledForOtherUser: Boolean = cuEnabledUser?.let { it.id != user.id } ?: false

            !hasCuEnabledForOtherUser && it.role == DbxUser.UserRole.PERSONAL &&
                    (isCUEnabledForUser(user) ||
                            isUserInEnableCapableState(user) || isUserInUpsellState(user))
        }
    }

    override fun shouldShowUpsellPromoBannerInHomeForUser(user: DbxUser?): Boolean =
            user != null &&
                    isUserInUpsellState(user) &&
                    !user.userProperties.cameraUploadPromoBannerDismissed() &&
                    ((shouldShowAllCuUIInHomeForUser(user) &&
                            isPersonalOrUnpairedBusinessUser(user)) ||
                            isShowUpsellPromoBannerOverrideEnabled(user))

    override fun shouldShowEnablePromoBannerInHomeForUser(user: DbxUser?): Boolean =
            user != null &&
                    isUserInEnableCapableState(user) &&
                    !user.userProperties.cameraUploadPromoBannerDismissed() &&
                    ((shouldShowAllCuUIInHomeForUser(user) &&
                            isPersonalOrUnpairedBusinessUser(user)) ||
                            isShowEnablePromoBannerOverrideEnabled(user))

    override fun shouldShowAdminDisabledBannerInHomeForUser(user: DbxUser?): Boolean =
            user != null &&
                    user.role == DbxUser.UserRole.BUSINESS &&
                    getUserCuAvailability(user) == UserCapability.Availability.NEVER_AVAILABLE &&
                    !isCameraUploadsStormcrowNeverAvailable(user) &&
                    user.userProperties.shouldShowAdminDisabledBannerInHome()

    override fun shouldShowCuStatusUIInHomeForUser(user: DbxUser?): Boolean =
            user != null && shouldShowAllCuUIInHomeForUser(user) && isCUEnabledForUser(user) &&
                    !user.userProperties.hasDismissedCameraUploadStatus

    private fun isUserInUpsellState(user: DbxUser?): Boolean =
            user != null &&
                    mayEnableCuIfCapabilityIsAvailableForUser(user) &&
                    getUserCuAvailability(user) ==
                    UserCapability.Availability.CAN_BECOME_AVAILABLE

    private fun isShowUpsellPromoBannerOverrideEnabled(user: DbxUser): Boolean = try {
            user.stormcrow
                    .isInVariantLogged(
                            StormcrowMobileAndroidShowCuUpsellBannerInHome.VENABLED)
        } catch (e: DbxException) { false }

    private fun isShowEnablePromoBannerOverrideEnabled(user: DbxUser): Boolean = try {
        user.stormcrow
                .isInVariantLogged(
                        StormcrowMobileAndroidShowCuEnableBannerInHome.VENABLED)
    } catch (e: DbxException) { false }

    private fun isCameraUploadsStormcrowNeverAvailable(user: DbxUser): Boolean = try {
        user.stormcrow
                .isInVariantLogged(
                        StormcrowMobileAndroidCameraUpload.VNEVER_AVAILABLE
                )
    } catch (e: DbxException) { false }

    private fun shouldShowAllCuUIInHomeForUser(user: DbxUser): Boolean =
            user.role == DbxUser.UserRole.BUSINESS

    private fun isUserInEnableCapableState(user: DbxUser): Boolean =
            mayEnableCuIfCapabilityIsAvailableForUser(user) &&
                    getUserCuAvailability(user) == UserCapability.Availability.AVAILABLE &&
                    !isCUEnabledForUser(user)

    /**
     * Whether the user is allowed to enable CU if their account has the capability.
     * Before we enable account switching, a user may not be able to enable CU even if they are the
     * most capable account.
     * TODO(weilhuiz): Delete this method entirely once StormcrowMobileAndroidCameraUploadAccountSwitching
     * is enabled for all users. At that point, this method will always return true
     */
    private fun mayEnableCuIfCapabilityIsAvailableForUser(user: DbxUser): Boolean {
        val isCuAccountSwitchingEnabled = try {
            user.stormcrow
                    .isInVariantLogged(
                            StormcrowMobileAndroidCameraUploadAccountSwitching.VENABLED)
        } catch (e: DbxException) {
            false
        }

        val isCuUser = cuCandidateId == user.id

        return (isCuUser || isCuAccountSwitchingEnabled) &&
                getUserCuAvailability(user) != UserCapability.Availability.NEVER_AVAILABLE
    }

    private fun getUserCuAvailability(user: DbxUser): UserCapability.Availability =
        user.capabilitiesManager.capabilityAvailability(UserCapability.CameraUpload::class.java)

    private fun isPersonalOrUnpairedBusinessUser(user: DbxUser): Boolean =
        user.role != DbxUser.UserRole.BUSINESS || !user.isPaired

    private fun isCUEnabledForUser(user: DbxUser): Boolean =
            user.userProperties.cameraUploadEnabled()
}
