package com.dropbox.android.camerauploads

import android.database.Cursor
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.RuntimePermissionPrompter
import com.dropbox.base.analytics.CameraUploadsFeatureEvents
import com.dropbox.base.error.DbxException
import com.dropbox.base.observer.Listeners
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsStatusSnapshot

/**
 * Interface for manipulating the Camera Uploads library. This will be either backed by a legacy
 * implementation of the CU library (implemented in the [CameraUploadService])
 * or new C++ library.
 */
/*package*/ internal interface CameraUploadsUserController {
    /**
     * @return True if CU is enabled for the user associated with this controller, false otherwise.
     */
    val isEnabled: Boolean

    /**
     * Get a cursor with information that reflects the current state of camera uploads, for
     * use in the status bar show in the PhotosGridFragment. This cursor will always be nonnull,
     * but may contain zero rows if there should be no status bar visible. If the cursor does
     * have a row, it contains the following columns:
     *
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_LEGACY_CURRENT_UPLOAD_ID_FOR_STATUS_PATH_CREATION]
     * - Integer column for legacy CU with [com.dropbox.android.taskqueue.UploadQueue] id
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_STATUS]
     * - String representation of a [com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus]
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_INITIAL_SCAN]
     * - Integer representation of a boolean. True if the library is doing its initial scan -
     * currently always false for new CU
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_NUM_REMAINING]
     * - Integer column with number of photos / videos pending upload
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_CURRENT_UPLOAD_LOCAL_URI]
     * - String represntation of a content Uri for the current upload. Null if no current upload.
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_CURRENT_UPLOAD_IS_VIDEO]
     * - Integer representation of a boolean. True if the current upload is a video,
     * false if it's a photo or no there isn't a current upload.
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_CURRENT_UPLOAD_PROGRESS_PERCENT]
     * - Integer column with percent progress (value 0 - 100) of current upload, or 0 if no upload.
     * [CameraUploadsStatusBarCursorHelper.CAMERA_UPLOAD_PATHS_TO_THUMB_JSON]
     * - String column containing JSONArray of content Uris serialized to String. These Uris
     * are pending uploads that need thumbs preloaded.
     */
    val statusCursor: Cursor

    /**
     * @return Current camera uploads status snapshot for this user
     */
    /**
     * @VisibleForTesting
     * Expose a method to set a snapshot. Useful only for testing
     */
    var currentViewModelSnapshot: DbxCameraUploadsStatusSnapshot

    /**
     * @return User ID for the user associated with this controller.
     */
    fun userId(): String

    /**
     * Enables camera uploads for the user associated with this controller, first prompting for
     * storage permissions if needed.
     * @param permissionRequester
     * @param uploadExisting whether CU should upload the existing photos
     * @param use3g whether CU should use 3G
     * @param uploadVideos whether CU should upload videos
     * @param successCallback an optional callback when CU was enabled successfully. Will be called
     */
    fun enableCameraUploadsIfPermissionGranted(
        permissionRequester: RuntimePermissionPrompter,
        source: CameraUploadsFeatureEvents.CameraUploadsEnableSource,
        uploadExisting: Boolean,
        use3g: Boolean,
        uploadVideos: Boolean,
        successCallback: Runnable?
    )

    /**
     * Disables camera uploads for the user associated with this controller, if it is not already
     * disabled.
     * @param source Indicates reason for disable. The source is internally mapped into whether
     *               CU was disabled by a user action or by the application.
     */
    fun disableCameraUploadsIfEnabled(source: CameraUploadsFeatureEvents.CameraUploadsDisableSource)

    /**
     * Starts up the CU engine and queues a scan if the user associated with this controller
     * has the feature enabled.
     */
    fun startupIfEligibleAndEnabled()

    /**
     * This is the main entry point for scheduling a job to perform CU scan and upload tasks for
     * the user that currently has CU enabled, e.g. scheduling a job after a media store change has
     * been detected
     * @param reason one of @{link EnqueueJobReason} to describe why does the scan and
     * upload job was scheduled
     */
    fun enqueueScanAndUploadJob(reason: CameraUploadJobManager.EnqueueJobReason)

    /**
     * This is the entry point to kick off the actual CU logic for scanning and uploading photos.
     * It should be invoked by a scheduled CU job in order to ensure running time, especially
     * when app is executing in the background.
     *
     * @param onUploadsCompleteCallback A callback that will run when this job completes. This
     * callback is guaranteed to run, even if the job doesn't start.
     *
     * @return true if the scan and upload has started correctly, false otherwise
     */
    fun startScanAndUploadJob(onUploadsCompleteCallback: Runnable?): Boolean

    /**
     * Perform corresponding clean-ups, such as clearing pending listeners or database, etc.
     */
    fun destroy()

    /**
     * Register a listener to act on DbxCameraUploadsStatusSnapshot changes. This listener will
     * also refresh when CU gating / user capabilities change.
     * @param listener A listener on CU user status changes
     */
    fun registerStatusSnapshotListener(
        snapshotListener: CameraUploadsManager.Listener
    ): Listeners.Registration

    /**
     * NOTE(ageiduschek): We won't need this once we no longer need to update the cuUser (i.e.
     * when we have released biz CU to 100%.
     */
    interface CameraUploadsGatingListener {
        /**
         * Triggered when a user's access to the CU feature has changed.
         */
        fun onCameraUploadsGatingChanged(user: DbxUser)
    }

    /**
     * Register a listener on CU gating changes for this user. Will be triggered when a user's access
     * to the CU feature has changed.
     *
     * NOTE(ageiduschek): We won't need this once we no longer need to update the cuUser (i.e.
     * when we have released biz CU to 100%.
     */
    fun registerCameraUploadsGatingListener(
        gatingListener: CameraUploadsGatingListener
    ): Listeners.Registration

    /**
     * Whether or not we should upload on cell data.
     */
    fun setUse3G(use3g: Boolean)

    /**
     * Method allowing to set a preference to turn CU off whenever device is not charging
     *
     * @param enabledOnlyWhenCharging - whether to allow Camera Upload only when device is charging.
     */
    fun setEnabledOnlyWhenCharging(enabledOnlyWhenCharging: Boolean)

    /**
     * Method allowing to set an arbitrary threshold battery level for turning CU off
     *
     * @param batteryThreshold number from range
     * [{@value com.dropbox.android.preference.BatteryLevelThresholdSeekBarDialogPreference#MINIMUM_VALUE} -
     * {@value com.dropbox.android.preference.BatteryLevelThresholdSeekBarDialogPreference#MAXIMUM_VALUE}]
     * indicating the threshold battery level that will trigger turning CU off.
     */
    fun setBatteryThreshold(batteryThreshold: Long)

    /**
     * Sets whether to allow video uploads using data plan.
     * @param limitVideoUploadsOverCell - if true the videos won't be uploaded over the data plan
     */
    fun setLimitVideoUploadsOverCell(limitVideoUploadsOverCell: Boolean)

    /**
     * Enables or disables video upload.
     */
    fun setVideosEnabled(videosEnabled: Boolean)

    /**
     * Force update the feature status and propagate it.
     */
    fun refreshSnapshotListeners()

    /**
     * Clear the CU Consistency Checker states to enable a fresh check.
     * It's only used for internal build and CU V3
     */
    @Throws(DbxException::class)
    fun debugResetCuConsistencyCheckerStates()

    /**
     * Clear the CU Consistency Checker states and start a check immediately.
     * It's only used for internal build and CU V3
     */
    @Throws(DbxException::class)
    fun debugResetAndStartCuConsistencyChecker()

    /**
     * TODO(ageiduschek): Delete this once the Android C++ integration is out to 100%
     */
    fun requireHashUpdate()
}
