/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.camerauploads

import android.content.Context
import android.os.Handler
import com.dropbox.android.camerauploads.CameraUploadsManager.DEFAULT_BATTERY_THRESHOLD_VALUE
import com.dropbox.android.notifications.SystemTrayNotifications
import com.dropbox.android.settings.GlobalProperties
import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.UserCapabilitiesManager
import com.dropbox.android.user.UserCapability
import com.dropbox.android.util.LogTagUtils
import com.dropbox.android.util.RuntimePermissionPrompter
import com.dropbox.base.analytics.CameraUploadsFeatureEvents
import com.dropbox.base.observer.Listeners
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsStatusSnapshot
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsViewModel
import java.util.concurrent.Executor

internal abstract class CameraUploadsUserControllerBase(
    protected val context: Context,
    protected val user: DbxUser,
    protected val jobManager: CameraUploadJobManager,
    protected val executor: Executor,
    private val mainThreadHandler: Handler
) : CameraUploadsUserController, UserCapabilitiesManager.UserCapabilityListener {

    override var currentViewModelSnapshot: DbxCameraUploadsStatusSnapshot =
            DbxCameraUploadsViewModel.defaultSnapshotForStatus(
                    DbxCameraUploadsFeatureStatus.NOT_STARTED)
        set(snapshot) {
            field = snapshot

            mainThreadHandler.post {
                if (snapshot.isControllerIdle) {
                    jobManager.setAllJobsFinished(user)
                }

                // If the user now has pending uploads, unset the bit set by the user to temporarily
                // hide the Cu status bar.
                if (snapshot.numUserPendingUploads > 0 || snapshot.numPhotosWithSyncErrors > 0) {
                    user.userProperties.hasDismissedCameraUploadStatus = false
                }

                notifyListenersWithCurrentSnapshot()
            }
        }

    private val snapshotListeners = Listeners.forUI<CameraUploadsManager.Listener>()
    private val gatingListeners =
            Listeners.forUI<CameraUploadsUserController.CameraUploadsGatingListener>()
    private var userCapabilityListenerRegistration: Listeners.Registration? = null

    companion object {
        val TAG = LogTagUtils.create(CameraUploadsUserControllerBase::class.java.simpleName)
    }

    init {
        executor.execute {
            userCapabilityListenerRegistration =
                    user.capabilitiesManager.addListener(
                            UserCapability.CameraUpload::class.java,
                            this)
        }
    }

    /**
     * Starts the CU engine.
     */
    abstract fun start()

    /**
     * Stops the CU engine.
     */
    abstract fun stop()

    /**
     * Tell the CU engine to try to start uploading again.
     */
    abstract fun kick()

    /**
     * Notify whenever the CU setting has been changed by the user.
     */
    protected abstract fun notifySettingsChanged()

    /**
     * TODO(weilhuiz): Deprecate this method in favor of notifySettingsChanged post legacy CU
     * Notify the CU engine that video uploads are enabled / disabled.
     * @param enabled whether video uploads are enabled
     */
    protected abstract fun notifyVideoUploadsEnabled(enabled: Boolean)

    override fun onCapabilityAvailabilityChanged(
        userId: String,
        capability: Class<out UserCapability>,
        oldAvailability: UserCapability.Availability,
        newAvailability: UserCapability.Availability
    ) {
        mainThreadHandler.post {
            if (newAvailability != UserCapability.Availability.AVAILABLE) {
                var disableSource =
                        CameraUploadsFeatureEvents
                        .CameraUploadsDisableSource.USER_CAPABILITY_CHANGE
                user.accountInfoManager.accountInfo?.let {
                    if (it.isCameraUploadsDisabledByAdmin) {
                        disableSource =
                                CameraUploadsFeatureEvents
                                .CameraUploadsDisableSource.TEAM_ADMIN_DISABLE
                    }
                }
                disableCameraUploadsIfEnabled(disableSource)
                user.userProperties.setCameraUploadPromoBannerDismissed(false)
                user.accountInfoManager.accountInfo?.let {
                    if (!it.isCameraUploadsDisabledByAdmin) {
                        user.userProperties.setShouldShowAdminDisabledBannerInHome(false)
                    }
                }
            }

            notifyListenersWithCurrentSnapshot()

            gatingListeners.dispatch { it.onCameraUploadsGatingChanged(user) }
        }
    }

    override fun startupIfEligibleAndEnabled() {
        jobManager.start(user)
        jobManager.enqueueJob(user, CameraUploadJobManager.EnqueueJobReason.APP_STARTED)
    }

    override fun registerStatusSnapshotListener(
        snapshotListener: CameraUploadsManager.Listener
    ): Listeners.Registration {
        val registration = snapshotListeners.register(snapshotListener)
        notifyListenersWithCurrentSnapshot()
        return registration
    }

    override fun registerCameraUploadsGatingListener(
        gatingListener: CameraUploadsUserController.CameraUploadsGatingListener
    ): Listeners.Registration = gatingListeners.register(gatingListener)

    override fun setUse3G(use3g: Boolean) {

        val userProps = user.userProperties

        if (userProps.cameraUploadUse3G() != use3g) {
            userProps.setCameraUploadUse3G(use3g)
            notifySettingsChanged()
        }
    }

    override fun setEnabledOnlyWhenCharging(enabledOnlyWhenCharging: Boolean) {
        val userProps = user.userProperties

        if (userProps.cameraUploadOnlyWhenCharging() != enabledOnlyWhenCharging) {
            userProps.setCameraUploadOnlyWhenCharging(enabledOnlyWhenCharging)
            notifySettingsChanged()
        }
    }

    override fun setBatteryThreshold(batteryThreshold: Long) {
        val userProps = user.userProperties

        if (userProps.cameraUploadBatteryThreshold() != batteryThreshold) {
            userProps.setCameraUploadBatteryThreshold(batteryThreshold)
            notifySettingsChanged()
        }
    }

    override fun setLimitVideoUploadsOverCell(limitVideoUploadsOverCell: Boolean) {
        val userProps = user.userProperties

        if (userProps.cameraUploadLimitVideoUploadsOverCell() != limitVideoUploadsOverCell) {
            userProps.setCameraUploadLimitVideoUploadsOverCell(limitVideoUploadsOverCell)
            notifySettingsChanged()
        }
    }

    override fun setVideosEnabled(videosEnabled: Boolean) {
        val userProps = user.userProperties

        if (userProps.cameraUploadVideosEnabled() != videosEnabled) {
            userProps.setCameraUploadVideosEnabled(videosEnabled)
            // Send the new setting to the server asynchronously.
            asyncSendVideoUploadEnabledSetting(user, videosEnabled)
            if (userProps.cameraUploadEnabled()) {
                notifyVideoUploadsEnabled(videosEnabled)
                if (videosEnabled) {
                    jobManager.enqueueJob(
                            user, CameraUploadJobManager.EnqueueJobReason.VIDEO_UPLOAD_ENABLED)
                }
            }
        }
    }

    override fun enqueueScanAndUploadJob(reason: CameraUploadJobManager.EnqueueJobReason) {
        jobManager.enqueueJob(user, reason)
    }

    override fun startScanAndUploadJob(onUploadsCompleteCallback: Runnable?): Boolean {
        if (!jobManager.setJobStarted(user, onUploadsCompleteCallback)) {
            DbxLog.external(TAG, "Can't start CU scan and upload operation. Returning.")
            return false
        }

        start()

        // Bump the queue in case it's currently blocking for failures, to make it retry
        // TODO(weilhuiz): We should be able to delete this when old CU goes away
        kick()
        return true
    }

    override fun destroy() {
        executor.execute {
            userCapabilityListenerRegistration?.unregister()
        }
    }

    protected fun notifyListenersWithCurrentSnapshot() {
        mainThreadHandler.post {
            snapshotListeners.dispatch {
                it.onCuSnapshotRefreshedForUser(user, currentViewModelSnapshot)
            }
        }
    }

    /**
     * A helper method to enable camera uploads, first prompting for storage permissions if needed.
     * @param runtimePermissionsHelper for requesting the WRITE_PHOTOS permission
     * @param uploadExisting whether CU should upload the existing photos
     * @param use3g whether CU should use 3G
     * @param uploadVideos whether CU should upload videos
     * @param successCallback an optional callback when CU was enabled successfully. Will be called
     *                        with a boolean parameter which is true if CU wasn't enabled previously
     *                        and the user setting was changed.
     */
    protected fun enableCameraUploadsForUserIfPermissionGranted(
        runtimePermissionsHelper: RuntimePermissionPrompter,
        source: CameraUploadsFeatureEvents.CameraUploadsEnableSource,
        uploadExisting: Boolean,
        use3g: Boolean,
        uploadVideos: Boolean,
        successCallback: Runnable?
    ) {
        // Ensure the user has storage permissions
        runtimePermissionsHelper
                .process(
                        object : RuntimePermissionPrompter.Callback {
                            override fun permissionGranted() {
                                enableCameraUploadsForUser(
                                        source,
                                        uploadExisting,
                                        use3g,
                                        uploadVideos)

                                successCallback?.run()
                            }

                            override fun permissionDenied() {}
                        })
    }

    /**
     * Disables camera uploads for the given user, if it is not already disabled.
     * @param turnedOffByApp Indicates whether CU was turned off by the app (true) or whether the
     *                      user chose to disable (false)
     * @return Whether CU was previously enabled and the CU setting was changed
     */
    protected fun disableCameraUploadsForUserIfEnabled(
        source: CameraUploadsFeatureEvents.CameraUploadsDisableSource
    ): Boolean {
        val didUpdate = updateCameraUploadEnabledSettingForUser(enabled = false)

        if (didUpdate) {
            jobManager.setAllJobsFinished(user)
            jobManager.stop(user)
            val trayNotificationController = user.systemTrayNotificationController
            // Clear any camera upload notifications since we're turning off.
            trayNotificationController.clear(
                    user.id, SystemTrayNotifications.Singular.CAMERA_UPLOAD_STOPPED_QUOTA)
            trayNotificationController.clear(
                    user.id, SystemTrayNotifications.Singular.CAMERA_UPLOAD_PAUSED_BATTERY)
            trayNotificationController.clear(
                    user.id, SystemTrayNotifications.Singular.CAMERA_UPLOAD_PAUSED_UNPLUGGED)

            user.logger.log(CameraUploadsFeatureEvents.Disabled().setDisableSource(source))

            val turnedOffByApp = cameraUploadsDisableSourceToTurnedOffByApp(source)
            if (turnedOffByApp) {
                user.userProperties.shouldShowCameraUploadTurnedOffModal = true

                user.accountInfoManager.accountInfo?.let {
                    if (it.isCameraUploadsDisabledByAdmin) {
                        user.userProperties.setShouldShowAdminDisabledBannerInHome(true)
                    }
                }
                // Show the notification
                trayNotificationController
                        .show(user.id,
                                SystemTrayNotifications.Singular.CAMERA_UPLOAD_TURN_OFF_NOTICE,
                                null)
                user.logger.log(CameraUploadsFeatureEvents.TurnedOffByApp())
            }
        }

        return didUpdate
    }

    private fun cameraUploadsDisableSourceToTurnedOffByApp(
        source: CameraUploadsFeatureEvents.CameraUploadsDisableSource
    ): Boolean {
        when (source) {
            CameraUploadsFeatureEvents
                    .CameraUploadsDisableSource.CU_USER_SWITCHED_UPLOADING_ACCOUNT,
            CameraUploadsFeatureEvents
                    .CameraUploadsDisableSource.USER_SETTINGS -> return false
            CameraUploadsFeatureEvents
                    .CameraUploadsDisableSource.CU_USER_CHANGED,
            CameraUploadsFeatureEvents
                    .CameraUploadsDisableSource.USER_CAPABILITY_CHANGE,
            CameraUploadsFeatureEvents
                    .CameraUploadsDisableSource.TEAM_ADMIN_DISABLE -> return true
            else -> {
                throw Exception("Unknown source")
            }
        }
    }

    private fun enableCameraUploadsForUser(
        source: CameraUploadsFeatureEvents.CameraUploadsEnableSource,
        uploadExisting: Boolean,
        use3g: Boolean,
        uploadVideos: Boolean
    ) {
        val userProps = user.userProperties

        userProps.setCameraUploadPromoBannerDismissed(true)
        userProps.setCameraUploadUse3G(use3g)
        userProps.setCameraUploadVideosEnabled(uploadVideos)
        // Send the new setting to the server asynchronously.
        asyncSendVideoUploadEnabledSetting(user, uploadVideos)

        userProps.setCameraUploadBatteryThreshold(DEFAULT_BATTERY_THRESHOLD_VALUE.toLong())
        userProps.setCameraUploadLimitVideoUploadsOverCell(true)

        // START legacy CU user settings
        // TODO(ageiduschek) Remove these once we roll CU V3 out to 100%
        userProps.setCameraUploadFirstMediaScan(true)
        userProps.setCameraUploadInitialScan(true)
        userProps.setCameraUploadIgnoreExisting(!uploadExisting)
        userProps.setCameraUploadHashUpdate(GlobalProperties.HashUpdate.OPTIONAL)
        userProps.setCameraUploadNotification(
                if (uploadExisting) GlobalProperties.CUNotification.AFTER_BACKLOG
                else GlobalProperties.CUNotification.AFTER_FIRST_SEVERAL)
        userProps.resetCameraUploadNumUploads()
        userProps.setCameraUploadHadBacklog(false)
        // END legacy CU user settings

        val didUpdate = updateCameraUploadEnabledSettingForUser(enabled = true)

        if (didUpdate) {
            jobManager.start(user)
            jobManager.enqueueJob(user, CameraUploadJobManager.EnqueueJobReason.SETTING_ENABLED)

            user.logger.log(CameraUploadsFeatureEvents.Enabled().setEnableSource(source))
        }
    }

    /**
     * @return True if the user exists and the setting changed, false otherwise.
     */
    private fun updateCameraUploadEnabledSettingForUser(
        enabled: Boolean
    ): Boolean {
        val userProperties = user.userProperties
        // Do nothing if there is no change
        if (userProperties.cameraUploadEnabled() == enabled) {
            return false
        }

        userProperties.setCameraUploadEnabled(enabled)

        return true
    }

    /**
     * Make a best-effort attempt to send the new video enabled setting to the server to be used for
     * future defaults. The api call happens in another thread, so this is safe to call from any thread
     * and returns immediately without checking for success. There is no retrying in case of failure.
     */
    private fun asyncSendVideoUploadEnabledSetting(user: DbxUser, videosEnabled: Boolean) {
        // Try to send the setting in new thread.
        object : Thread() {
            override fun run() {

                try {
                    user.api.setVideoUploadsEnabled(videosEnabled)
                } catch (e: DropboxException) {
                    // Fail silently
                    DbxLog.externalError("CameraUploadsUserControllerBase",
                            "Synching video uploads default to server failed.", e)
                }
            }
        }.start()
    }
}
