/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.camerauploads

import com.dropbox.android.user.DbxUser
import com.dropbox.android.user.DbxUserManager
import com.dropbox.android.user.DbxUserManager.OnUsersetChangedMainThreadListener
import com.dropbox.android.user.DbxUserset
import com.dropbox.core.stormcrow.StormcrowMobileAndroidCameraUploadAccountSwitching
import com.dropbox.base.analytics.CameraUploadsFeatureEvents
import java.util.HashMap

/**
 * Responsible for creating/destorying and listening on camera uploads user controllers as
 * the userset changes.
 */
internal interface CameraUploadsUserControllerManager {
    /**
     * Initializes the controller manager. Must be called after the userset has loaded.
     */
    fun initialize()

    /**
     * Returns the controller for the enabled user, or null if there are no users with CU enabled
     */
    val enabledUserController: CameraUploadsUserController?

    /**
     * Returns the controller for the given user, or null if there is no user with the given id.
     */
    fun controllerForUser(userId: String): CameraUploadsUserController?
}

internal class CameraUploadsUserControllerManagerImpl(
    private val userManager: DbxUserManager,
    private val userControllerFactory: CameraUploadsUserControllerFactory,
        // TODO(ageiduschek): Delete this once we've enabled account switching as part of Biz CU and
        // we no longer reference the "CU user".
    private val cuCandidateChangeModel: CameraUploadsUserStateChangeModel
) : CameraUploadsUserControllerManager, OnUsersetChangedMainThreadListener,
        CameraUploadsUserStateChangeModel.Listener,
        CameraUploadsUserController.CameraUploadsGatingListener {

    private val userControllers = HashMap<String, CameraUploadsUserController>()

    /* CameraUploadsUserControllerManager implementation */

    override fun initialize() {
        refreshForUpdatedUserset()
        userManager.registerOnUsersetChangedMainThreadListener(this)
        cuCandidateChangeModel.registerListener(this)
        cuCandidateChangeModel.init(userManager.users)
    }

    override fun controllerForUser(userId: String): CameraUploadsUserController? {
        return userControllers[userId]
    }

    override val enabledUserController: CameraUploadsUserController?
        get() {
            val enabledUserId = userManager.users?.allUsers()?.firstOrNull { user ->
                user.userProperties.cameraUploadEnabled()
            }?.id
            return enabledUserId?.let { userControllers[it] }
        }

    /* Listener implementations */

    override fun onUsersetChangedInMainThread(newUserset: DbxUserset?) {
        refreshForUpdatedUserset()
        cuCandidateChangeModel.updateUserset(newUserset)
    }

    override fun onCameraUploadsGatingChanged(user: DbxUser) {
        // We only notify listeners here if something has changed in the state change
        // model (in which case `onCUCandidateUserChanged` will be fired. The user controller
        // is responsible for updating listeners for all other capability changes (and will have
        //  already done that by the time we get here).
        cuCandidateChangeModel.refreshBestUserAndNotifyIfNecessary()
    }

    // TODO(ageiduschek): Delete this once we've fully enabled account switching as part of Biz CU
    // and we no longer reference the "CU user".
    override fun onCUCandidateUserChanged(cuCandidateUser: DbxUser?) {
        if (!accountSwitchingEnabledFor(
                        userManager.users?.getUserById(enabledUserController?.userId()))) {
            enabledUserController?.let {
                // Always turn CU off for the previous CU User
                // TODO(ageiduschek): Only do this if the account switching stormcrow is disabled!
                // (Also make sure we make the same fix on iOS)
                if (cuCandidateUser == null || it.userId() !== cuCandidateUser.id) {
                    it.disableCameraUploadsIfEnabled(
                            CameraUploadsFeatureEvents.CameraUploadsDisableSource.CU_USER_CHANGED)
                }
            }
        }

        refreshSnapshotListeners()
    }

    /* Private helpers */

    private fun refreshForUpdatedUserset() {
        // Shutdown/delete controllers for users that have been removed
        userControllers.keys.filter {
            userManager.users?.getUserById(it) == null
        }.forEach {
            userControllers.remove(it)?.apply {
                disableCameraUploadsIfEnabled(
                        CameraUploadsFeatureEvents
                                .CameraUploadsDisableSource.CU_USER_CHANGED)
                destroy()
            }
        }

        // Create new controllers for new users
        userManager.users?.allUsers()?.filter {
            !userControllers.containsKey(it.id)
        }?.forEach {
            userControllers[it.id] = userControllerFactory.createForUser(it).also {
                it.registerCameraUploadsGatingListener(this)
            }
        }

        refreshSnapshotListeners()
    }

    // NOTE(weilhuiz): After certain user state changes we need to force update the status as the
    // Photos tab doesn't query the new feature status automatically, so it may display a wrong state.
    private fun refreshSnapshotListeners() {
        userControllers.values.forEach {
            controller -> controller.refreshSnapshotListeners()
        }
    }

    private fun accountSwitchingEnabledFor(user: DbxUser?): Boolean {
        return user?.stormcrow?.isInVariantLogged(
                StormcrowMobileAndroidCameraUploadAccountSwitching.VENABLED) ?: false
    }
}
