// dbx-min-coverage: 22% because unable to test the rest
package com.dropbox.android.cloudmessaging

import com.dropbox.android.exception.EnvInfo
import com.dropbox.android.notifications.FreshNotificationManager
import com.dropbox.android.user.DbxUserManager
import com.dropbox.android.user.DbxUserset
import com.dropbox.base.analytics.Analytics
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.base.rxjava_utils.DbxObservable
import com.dropbox.base.rxjava_utils.RetryWhenObservable
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.legacy_api.exception.DropboxIOException
import com.dropbox.core.legacy_api.exception.DropboxServerException
import com.dropbox.core.legacy_api.exception.DropboxUnlinkedException
import io.reactivex.Scheduler

private val TAG = DbxCloudMessagingManager::class.java.name
private const val ANALYTICS_FAILURE_EVENT_NAME = "subscribe_error"

/**
 * Manages the updates of the registration token for each logged in user.
 */
class DbxCloudMessagingManager(
    private val firebaseFacade: DbxFirebaseFacade = DbxFirebaseFacade(),
    private val userManager: DbxUserManager,
    private val analyticsLogger: AnalyticsLogger,
    private val envInfo: EnvInfo,
    private val subscribeOn: Scheduler
) {

    /**
     * Initializes the manager to listen to user changes
     */
    fun initManager() {
        userManager.registerOnUsersetChangedBackgroundThreadListener {
            firebaseFacade.getInstanceId().addOnSuccessListener { instanceIdResult ->
                val instanceToken = instanceIdResult.token
                DbxLog.d(TAG, "Userset has changed: using token $instanceToken")
                sendRegistrationToServer(instanceToken)
            }
        }

        firebaseFacade.getInstanceId().addOnSuccessListener { instanceIdResult ->
            val currentToken = instanceIdResult.token
            DbxLog.d(TAG, "Current token: $currentToken")

            sendRegistrationToServer(currentToken)
        }
    }

    /**
     * Updates the registration token for each logged in user.
     */
    internal fun sendRegistrationToServer(token: String?) {
        val users = userManager.users
        DbxLog.d(TAG, "Ensuring subscribed: ${DbxUserset.getUserIds(users)}")

        if (users != null) {
            for (user in users.allUsers()) {
                DbxObservable.observe { user.api.subscribeNotifications(token, envInfo.deviceId) }
                        // This only logs/intercepts error but not actually handle it in current Rx version
                        .doOnError(subscribeErrorHandler())
                        .retryWhen(RetryWhenObservable(maxAttempts = 10, scheduler = subscribeOn))
                        .subscribeOn(subscribeOn)
                        // Pass empty onError here as we don't have actions upon other than
                        // swallow the errors return from server after 10 attempts.
                        // Ignore and give up here if we fail to subscribe user even after 10 tries, it
                        // means there is some issues from server side and the user is unable to receive push notifications in
                        // Will retry when listeners in initManager() are triggered
                        .subscribe(
                            { Analytics.FCMEvent("success").logTo(analyticsLogger) }, {})
            }
        }
    }

    private fun subscribeErrorHandler(): (Throwable) -> Unit = fun(exception: Throwable) {
        DbxLog.d(TAG, exception.localizedMessage, exception)

        when (exception) {
            is DropboxUnlinkedException ->
                Analytics.FCMEvent(ANALYTICS_FAILURE_EVENT_NAME)
                    .set("unlink_err", exception.toString())
                    .logTo(analyticsLogger)
            is DropboxIOException ->
                Analytics.FCMEvent(ANALYTICS_FAILURE_EVENT_NAME)
                    .set("network_err", exception.toString())
                    .logTo(analyticsLogger)
            is DropboxServerException ->
                if (exception.error < 500) {
                    // We don't expect any 4xx or 3xx errors when subscribing.
                    Analytics.FCMEvent(ANALYTICS_FAILURE_EVENT_NAME)
                            .set("server_err", exception.toString())
                            .logTo(analyticsLogger)
                }
            is DropboxException ->
                Analytics.FCMEvent(ANALYTICS_FAILURE_EVENT_NAME)
                        .set("unknown_err", exception.toString())
                        .logTo(analyticsLogger)
        }
    }

    internal fun triggerNotificationsFetch(userId: String) {
        val users = userManager.users
        if (users != null) {
            val user = users.getUserById(userId)
            user?.freshNotificationManager?.apply {
                DbxObservable.observe(triggerNotifications(userId))
                        .subscribeOn(subscribeOn)
                        .doOnError {
                            DbxLog.d(TAG, it.localizedMessage, it)
                            Analytics.FCMEvent("update_fail")
                                .set("err", it.localizedMessage)
                                .logTo(analyticsLogger) }
                        .subscribe()
            }
        }
    }

    private fun FreshNotificationManager.triggerNotifications(userId: String): () -> Unit {
        return {
            DbxLog.d(TAG, "Dispatching updateFeed for user id $userId")

            triggerNotificationsBlockingFetch()
            Analytics.FCMEvent("update")
                    .set("trigger", "fcm")
                    .logTo(analyticsLogger)
        }
    }
}
