/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.content.camerauploads

import android.content.Context
import com.dropbox.android.R
import com.dropbox.android.camerauploads.CameraUploadsGatingManager
import com.dropbox.android.content.ViewModel
import com.dropbox.android.content.ViewModelsGenerator
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.UpgradabilityGate
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsStatusSnapshot

/**
 * Generator for {@link CameraUploadsViewModelProvider}. Handles the logic on how to generate a
 * {@link CameraUploadsViewModel}
 */
class CameraUploadsViewModelsGenerator(
    private val context: Context
) : ViewModelsGenerator() {
    var viewModels: List<ViewModel> = emptyList()
    private var viewModelCache: CameraUploadsViewModel? = null

    companion object {
        @JvmStatic
        fun getCuStatusTitle(
            context: Context,
            status: DbxCameraUploadsFeatureStatus,
            canEnableCu: Boolean
        ): String =
                when (status) {
                    DbxCameraUploadsFeatureStatus.REMOTE_PAUSED ->
                        context.getString(R.string.camera_upload_status_remote_paused_title)
                    DbxCameraUploadsFeatureStatus.DISABLED ->
                        if (canEnableCu) {
                            context.getString(R.string.camera_upload_prefs_turn_on)
                        } else {
                            context.getString(
                                    R.string.camera_upload_status_disabled_user_not_eligible_title)
                        }
                    DbxCameraUploadsFeatureStatus.ALL_DONE ->
                        context.getString(R.string.camera_upload_status_all_done_title)
                    DbxCameraUploadsFeatureStatus.NOT_STARTED,
                    DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY,
                    DbxCameraUploadsFeatureStatus.UPLOADING,
                    DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF,
                    DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS,
                    DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA,
                    DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING,
                    DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED,
                    DbxCameraUploadsFeatureStatus.NO_CONNECTION,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK,
                    DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
                    DbxCameraUploadsFeatureStatus.ROAMING_PAUSED,
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED,
                    DbxCameraUploadsFeatureStatus.PERMISSION_DENIED,
                    DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY,
                    DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED ->
                        context.getString(R.string.camera_upload_status_item_title)
                }

        /**
         * CU status bar subtitle for the given status. These strings are only shown if the user
         * is running CU v3. Returns null if no subtitle should be shown.
         */
        @JvmStatic
        fun getCuStatusSubtitle(
            context: Context,
            status: DbxCameraUploadsFeatureStatus,
            canEnableCu: Boolean,
            numPendingVideoUploads: Int
        ): String =
                when (status) {
                    DbxCameraUploadsFeatureStatus.DISABLED ->
                        if (canEnableCu) {
                            context.getString(R.string.camera_upload_status_disabled_subtitle)
                        } else {
                            context.getString(R.string
                                    .camera_upload_status_disabled_user_not_eligible_subtitle)
                        }
                    DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY ->
                        context.getString(R.string.camera_upload_status_finding_short)
                    DbxCameraUploadsFeatureStatus.ALL_DONE ->
                        context.getString(R.string.camera_upload_status_all_done_sub_title)
                    DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF -> {
                        context.resources.getQuantityString(
                                R.plurals.camera_upload_status_video_uploads_off,
                                numPendingVideoUploads,
                                numPendingVideoUploads)
                    }
                    DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS ->
                        context.getString(R.string.camera_upload_status_waiting_for_manual_uploads)
                    DbxCameraUploadsFeatureStatus.REMOTE_PAUSED ->
                        context.getString(R.string.camera_upload_status_remote_paused_subtitle)
                    DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA ->
                        // TODO(ageiduschek): Update this string to include a call to action,
                        // but only for users who have the CanIncreaseQuota capability (T172477)
                        context.getString(R.string.camera_upload_status_out_of_dropbox_quota)
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING ->
                        context.getString(R.string.camera_upload_status_low_battery_charging)
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING ->
                        context.getString(R.string.camera_upload_status_low_battery_not_charging)
                    DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED ->
                        context.getString(R.string.camera_upload_status_cannot_upload_unplugged)
                    DbxCameraUploadsFeatureStatus.NO_CONNECTION ->
                        context.getString(R.string.status_waiting_for_connection)
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI ->
                        context.getString(R.string.camera_upload_status_waiting_for_wifi)
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS ->
                        context.getString(R.string.camera_upload_status_waiting_for_wifi_for_videos)
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK ->
                        context.getString(R.string.camera_upload_status_waiting_for_faster_network)
                    DbxCameraUploadsFeatureStatus.ROAMING_PAUSED ->
                        context.getString(R.string.camera_upload_status_roaming_paused)
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED ->
                        context.getString(R.string.camera_upload_status_scan_failed)
                    DbxCameraUploadsFeatureStatus.PERMISSION_DENIED ->
                        context.getString(
                                R.string.camera_upload_permissions_denied_snackbar_message)
                        // We don't show a subtitle for this status
                    DbxCameraUploadsFeatureStatus.UPLOADING,
                        // Currently unused in CU v3
                    DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
                        // Ephemeral status, don't show a message
                    DbxCameraUploadsFeatureStatus.NOT_STARTED,
                        // Not used in CU v3
                    DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
                        // Currently unused on Android
                    DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY,
                    DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED -> ""
                }

        @JvmStatic
        fun getCuStatusSubtitleColor(status: DbxCameraUploadsFeatureStatus): Int =
                when (status) {
                    DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA ->
                        R.color.filelist_text_small_error
                    DbxCameraUploadsFeatureStatus.DISABLED,
                    DbxCameraUploadsFeatureStatus.NOT_STARTED,
                    DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY,
                    DbxCameraUploadsFeatureStatus.UPLOADING,
                    DbxCameraUploadsFeatureStatus.ALL_DONE,
                    DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF,
                    DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS,
                    DbxCameraUploadsFeatureStatus.REMOTE_PAUSED,
                    DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING,
                    DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED,
                    DbxCameraUploadsFeatureStatus.NO_CONNECTION,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK,
                    DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
                    DbxCameraUploadsFeatureStatus.ROAMING_PAUSED,
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED,
                    DbxCameraUploadsFeatureStatus.PERMISSION_DENIED,
                    DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
                    DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED ->
                        R.color.dbx_gray_stateful
                }

        @JvmStatic
        fun getCuStatusIcon(status: DbxCameraUploadsFeatureStatus): Int =
                when (status) {
                    DbxCameraUploadsFeatureStatus.ALL_DONE ->
                        R.drawable.icon_indicator_camera_blue
                    DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA,
                    DbxCameraUploadsFeatureStatus.REMOTE_PAUSED,
                    DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
                    DbxCameraUploadsFeatureStatus.NO_CONNECTION,
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED,
                    DbxCameraUploadsFeatureStatus.PERMISSION_DENIED ->
                        R.drawable.icon_indicator_camera_red
                    DbxCameraUploadsFeatureStatus.DISABLED,
                    DbxCameraUploadsFeatureStatus.NOT_STARTED,
                    DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY,
                    DbxCameraUploadsFeatureStatus.UPLOADING,
                    DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF,
                    DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS,
                    DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING,
                    DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK,
                    DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
                    DbxCameraUploadsFeatureStatus.ROAMING_PAUSED,
                    DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED ->
                        R.drawable.icon_indicator_camera_gray
                }

        @JvmStatic
        fun getControlIcon(
            cuSnapshot: DbxCameraUploadsStatusSnapshot
        ): CameraUploadsControlIcon =
                when (cuSnapshot.status) {
                    DbxCameraUploadsFeatureStatus.ALL_DONE -> CameraUploadsControlIcon.DISMISSAL
                    DbxCameraUploadsFeatureStatus.UPLOADING -> {
                        if (cuSnapshot.cuFolderPathRelativeToUserRoot != null) {
                            CameraUploadsControlIcon.GOTO_CU_FOLDER
                        } else {
                            CameraUploadsControlIcon.NONE
                        }
                    }
                    DbxCameraUploadsFeatureStatus.SCANNING_LIBRARY,
                    DbxCameraUploadsFeatureStatus.DISABLED,
                    DbxCameraUploadsFeatureStatus.NOT_STARTED,
                    DbxCameraUploadsFeatureStatus.VIDEO_UPLOADS_OFF,
                    DbxCameraUploadsFeatureStatus.PAUSED_FOR_PENDING_MANUAL_UPLOADS,
                    DbxCameraUploadsFeatureStatus.REMOTE_PAUSED,
                    DbxCameraUploadsFeatureStatus.OVER_DROPBOX_QUOTA,
                    DbxCameraUploadsFeatureStatus.OVER_BATTERY_QUOTA,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_CHARGING,
                    DbxCameraUploadsFeatureStatus.LOW_BATTERY_NOT_CHARGING,
                    DbxCameraUploadsFeatureStatus.CANNOT_UPLOAD_UNPLUGGED,
                    DbxCameraUploadsFeatureStatus.NO_CONNECTION,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_FOR_VIDEOS,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_WIFI_TO_SCAN_CLOUD_LIBRARY,
                    DbxCameraUploadsFeatureStatus.WAITING_FOR_FASTER_NETWORK,
                    DbxCameraUploadsFeatureStatus.WAITING_TO_UPLOAD,
                    DbxCameraUploadsFeatureStatus.ROAMING_PAUSED,
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED,
                    DbxCameraUploadsFeatureStatus.PERMISSION_DENIED,
                    DbxCameraUploadsFeatureStatus.NEED_SETTINGS_UPDATE,
                    DbxCameraUploadsFeatureStatus.ICLOUD_SYNC_PAUSED ->
                        CameraUploadsControlIcon.NONE
                }
    }

    fun updateViewModel(
        user: DbxUser,
        cuSnapshot: DbxCameraUploadsStatusSnapshot,
        cuGatingManager: CameraUploadsGatingManager
    ) {
        viewModelCache = CameraUploadsViewModel.Builder().injectFrom(viewModelCache).apply {
            val canEnableCu = cuGatingManager.canEnableCuForUser(user)
            cuUserStatus = CameraUploadsUserStatus(
                    cuSnapshot = cuSnapshot,
                    userCanUpgrade = UpgradabilityGate.isUserUpgradable(
                            context, user.capabilitiesManager),
                    userCanEnableCameraUploads = canEnableCu,
                    shouldShowConfirmTranscodeSettingBannerInHome = false,
                    shouldShowStatusBarInHome =
                    cuGatingManager.shouldShowCuStatusUIInHomeForUser(user),
                    shouldShowEnableBannerInHome =
                    cuGatingManager.shouldShowEnablePromoBannerInHomeForUser(user),
                    shouldShowUpsellBannerInHome =
                    cuGatingManager.shouldShowUpsellPromoBannerInHomeForUser(user),
                    cuControlIcon =
                    getControlIcon(cuSnapshot),
                    cuStatusTitle = getCuStatusTitle(
                            context = context,
                            status = cuSnapshot.status,
                            canEnableCu = canEnableCu),
                    cuStatusSubtitle = getCuStatusSubtitle(
                            context = context,
                            status = cuSnapshot.status,
                            canEnableCu = canEnableCu,
                            numPendingVideoUploads = cuSnapshot.numQueuedVideoUploads),
                    cuStatusSubtitleColor = getCuStatusSubtitleColor(status = cuSnapshot.status),
                    cuStatusIcon = getCuStatusIcon(status = cuSnapshot.status)
            )
        }.build()

        viewModelCache?.also {
            this.viewModels = listOf<CameraUploadsViewModel>(it)
        }
    }
}
