/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.content.camerauploads

import android.content.Context
import android.support.annotation.CallSuper
import android.support.annotation.UiThread
import com.dropbox.android.camerauploads.CameraUploadsManager
import com.dropbox.android.content.SingleThreadGroupFactory
import com.dropbox.android.content.ViewModelsProvider
import com.dropbox.android.content.inject.PresenterScope
import com.dropbox.android.user.DbxUser
import com.dropbox.android.util.RegistrationStack
import com.dropbox.base.oxygen.mainThreadOnly
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsViewModel
import javax.inject.Inject

/**
 * View models provider for {@link CameraUploadsViewModel}s.
 * It register the corresponding CU listeners to initiate the CU UI updates accordingly
 */
@PresenterScope
class CameraUploadsViewModelsProvider @Inject constructor(
    private val registrationStack: RegistrationStack,
    singleThreadGroupFactory: SingleThreadGroupFactory,
    private val cameraUploadsManager: CameraUploadsManager,
    private val user: DbxUser,
    context: Context
) : ViewModelsProvider(registrationStack, singleThreadGroupFactory) {

    private val generator = CameraUploadsViewModelsGenerator(context = context)
    private val updateViewModelsWork = UpdateViewModelsWork()
    private var lastCuSnapshot =
            DbxCameraUploadsViewModel.defaultSnapshotForStatus(
                    DbxCameraUploadsFeatureStatus.NOT_STARTED)

    @UiThread
    @CallSuper
    override fun initialize() {
        checkIsClosed()
        mainThreadOnly()

        super.initialize()

        registerCameraUploadsUserStateChangeListener()
        registerAccountInfoChangeListener()
        registerUserHasDismissCuStatusListener()
    }

    override fun queueUpdateViewModelsWork() {
        checkNotClosed()
        queueWork(updateViewModelsWork, true /*cancelWaitingInstances*/)
    }

    private fun registerCameraUploadsUserStateChangeListener() {
        cameraUploadsManager.registerCuUserStatusChangeForUser(user) { _, cuSnapshot ->
            mainThreadOnly()
            lastCuSnapshot = cuSnapshot
            queueUpdateViewModelsWork()
        }.also { registrationStack.track(it) }
    }

    /**
     * NOTE(weilhuiz) 07-23-2018: Due to the RegistrationStack will force any listener to
     * register/unregister on the main thread, we can't listen on the capability changes due to its
     * listeners are created in the background, therefore we will listen on the account info
     * change instead as it's the source for triggering the capability changes.
     */
    private fun registerAccountInfoChangeListener() {
        registrationStack.registerAndTrack(user.accountInfoManager) { _, _ ->
            if (!isClosed) {
                queueUpdateViewModelsWork()
            }
        }
    }

    private fun registerUserHasDismissCuStatusListener() {
        user.userProperties.addCameraUploadStatusDismissedListener {
            queueUpdateViewModelsWork()
        }.also { registrationStack.track(it) }
    }

    private inner class UpdateViewModelsWork : ViewModelsProvider.Work() {
        public override fun runWork() {
            generator.updateViewModel(user, lastCuSnapshot, cameraUploadsManager.gatingManager)
            postViewModels(generator.viewModels)
        }
    }
}
