// dbx-min-coverage: 0% because it's handling the UI binding logic
package com.dropbox.android.content.camerauploads.activity

import android.content.Context
import android.graphics.Typeface
import android.graphics.drawable.BitmapDrawable
import android.provider.MediaStore.Images
import android.support.v4.content.ContextCompat
import android.text.Html
import android.text.method.LinkMovementMethod
import android.view.View
import android.widget.ImageView
import com.dropbox.android.R
import com.dropbox.android.activity.BaseBrowserFragment
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.camerauploads.CameraUploadsManager
import com.dropbox.android.camerauploads.CameraUploadsUserStatusSelectionHelper
import com.dropbox.android.content.activity.ViewBinder
import com.dropbox.android.content.activity.ViewHolder
import com.dropbox.android.content.activity.ViewHolderType
import com.dropbox.android.content.camerauploads.CameraUploadsControlIcon
import com.dropbox.android.content.camerauploads.CameraUploadsViewModel
import com.dropbox.android.content.inject.PresenterScope
import com.dropbox.android.filemanager.LocalThumbManager
import com.dropbox.android.user.DbxUser
import com.dropbox.android.widget.ItemViewUtils
import com.dropbox.core.ui.widgets.listitems.DbxListItem
import com.dropbox.product.dbapp.camera_upload.cu_ui.DbxCameraUploadsFeatureStatus
import com.google.common.collect.ImmutableList
import javax.inject.Inject

/**
 * View binder for [com.dropbox.android.content.camerauploads.CameraUploadsViewModel].
 */
@PresenterScope
class CameraUploadsViewBinder @Inject internal constructor(
    val context: Context,
    val activity: BaseActivity,
    val user: DbxUser,
    val cameraUploadsManager: CameraUploadsManager,
    private val thumbManager: LocalThumbManager
) : ViewBinder() {

    private val cuUserStatusSelectionHelper = CameraUploadsUserStatusSelectionHelper(
            context = context,
            activity = activity,
            user = user,
            cameraUploadsManager = cameraUploadsManager)

    private var lastThumbRequest = 0

    override fun bindChildViews(viewHolder: ViewHolder) {
        check(viewHolder is CameraUploadsViewHolder) { "Cannot handle: $viewHolder" }

        with(viewHolder as CameraUploadsViewHolder) {
            bindStatusViewClickListener(view, viewModel)
            bindStatusIcon(cuListItem, viewModel)
            bindTitleView(cuListItem, viewModel)
            bindSubtitleView(cuListItem, viewModel)
            bindProgressBar(cuListItem, viewModel)
            bindStatusImage(cuListItem, viewModel)
            bindControlButton(cuListItem, viewModel)
        }
    }

    override fun unbindChildViews(viewHolder: ViewHolder) {
        check(viewHolder is CameraUploadsViewHolder) { "Cannot handle: $viewHolder" }

        unbindStatusViewClickListener(viewHolder.view)
        with(viewHolder as CameraUploadsViewHolder) {
            unbindControlButton(viewHolder.cuListItem)
        }
    }

    override fun getViewHolderTypes(): ImmutableList<ViewHolderType> {
        return ImmutableList.of(ViewHolderType.LIST_CAMERA_UPLOADS_VIEW_HOLDER)
    }

    private fun bindStatusViewClickListener(view: View, viewModel: CameraUploadsViewModel) {
        view.setOnClickListener {
            cuUserStatusSelectionHelper.onCUStatusBarClicked(viewModel.cuStatus)
        }
    }

    private fun bindStatusIcon(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        cuListItem.clearPrimaryIconOverlays()
        cuListItem.setPrimaryIcon(viewModel.cuStatusIcon)
        cuListItem.setPrimaryIconScaleType(ImageView.ScaleType.CENTER)
    }

    private fun bindTitleView(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        cuListItem.setTitleText(viewModel.cuStatusTitle)
        cuListItem.setTitleFontWeight(Typeface.BOLD)
    }

    private fun bindSubtitleView(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        cuListItem.subtitleText =
                when (viewModel.cuStatus) {
                    DbxCameraUploadsFeatureStatus.SCAN_FAILED -> {
                        // NOTE(weilhuiz): This is required as we need to parse the underlying
                        // HTML text to readable string. This behavior should be deprecated
                        // once we have a better error resolution flow for scan failed photos.
                        cuListItem.setSubtitleTextClickable(LinkMovementMethod.getInstance())
                        @Suppress("DEPRECATION")
                        Html.fromHtml(viewModel.cuStatusSubtitle)
                    } else -> {
                        viewModel.cuStatusSubtitle
                    }
                }

        ContextCompat.getColorStateList(context, viewModel.cuStatusSubtitleColor)?.let {
            cuListItem.setSubtitleTextColor(it)
        }

        cuListItem.setSubtitleTextScrollable()
    }

    /**
     * NOTE(weilhuiz): The ItemViewUtils.bindProgress operation will override the subtitle text.
     */
    private fun bindProgressBar(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        viewModel.uploadProgress?.let {
            // TODO(weilhuiz): Ideally we should bind the show/hide logic for the progress bar
            // programmatically rather than relying on the {@link DbxListItem} component
            ItemViewUtils.bindProgress(it, cuListItem)
        }
    }

    private fun bindStatusImage(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        viewModel.currentUploadURI?.let {
                thumbManager
                        .getThumb(it /*location*/,
                                lastThumbRequest++,
                                Images.Thumbnails.MICRO_KIND,
                                null /*callback*/)?.also { thumbManager ->
                                    ItemViewUtils.setThumbnail(
                                            cuListItem,
                                            BitmapDrawable(context.resources, thumbManager.bitmap),
                                            0 /*linkRes*/,
                                            0 /*starredRes*/,
                                            ItemViewUtils.getVideoIconOverlayResource(
                                                    viewModel.isCurrentUploadVideo),
                                            true,
                                            BaseBrowserFragment.DirectoryLayoutType.LIST)
                                }
        } ?: bindStatusIcon(cuListItem, viewModel)
    }

    private fun bindControlButton(cuListItem: DbxListItem, viewModel: CameraUploadsViewModel) {
        when (viewModel.cuControlIcon) {
            CameraUploadsControlIcon.DISMISSAL -> {
                cuListItem.setControlIcon(R.drawable.ic_close_grey_24dp)
                cuListItem.setControlIconOnClickListener {
                    cuUserStatusSelectionHelper.onCUStatusBarDismissed(viewModel.cuStatus)
                }
            }
            CameraUploadsControlIcon.GOTO_CU_FOLDER -> {
                cuListItem.setControlIcon(R.drawable.ic_arrow_forward_dbx_primary_24dp)
                cuListItem.setControlIconOnClickListener {
                    cuUserStatusSelectionHelper.onCUStatusBarClicked(viewModel.cuStatus)
                }
            }
            else -> {
                unbindControlButton(cuListItem)
            }
        }
    }

    private fun unbindStatusViewClickListener(view: View) {
        view.setOnClickListener(null)
    }

    private fun unbindControlButton(cuListItem: DbxListItem) {
        cuListItem.setControlIconOnClickListener(null)
        cuListItem.controlIcon = null
    }

    /* Ignore rebind as view model is the same */
    override fun rebindChildViews(viewHolder: ViewHolder) {}
}
