// dbx-min-coverage: 0% because grandfathered
package com.dropbox.android.devsettings

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RadioButton
import android.widget.RadioGroup
import com.dropbox.android.R
import com.dropbox.android.activity.StormcrowOverrideFragment
import com.dropbox.android.activity.base.BaseFragment
import com.dropbox.android.activity.dialog.TextProgressDialogFrag
import com.dropbox.android.util.AppInfoUtils
import com.dropbox.android.util.AppVersion
import com.dropbox.android.util.DebugConfig
import com.dropbox.android.util.SnackbarFactory
import com.dropbox.base.build_configuration.DevBuild
import com.dropbox.base.devsettings.BackendType
import com.dropbox.base.devsettings.DevSettings
import com.jakewharton.processphoenix.ProcessPhoenix
import kotlinx.android.synthetic.main.dev_settings_screen.*
import kotlinx.android.synthetic.main.dropbox_navigation_tab_content.view.dbx_toolbar
import org.apache.commons.lang3.StringUtils

private const val TAG = "DevSettingsFragment"

const val FRAG_TAG = "DevSettingsFragment_FRAG_TAG"

/**
 * Fragment for specifying server endpoints (prod vs dev vm) and test users. For internal developer use only.
 */
class DevSettingsFragment : BaseFragment() {

    private lateinit var devSettings: DevSettings
    private lateinit var previousBackendType: BackendType

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        devSettings = DebugConfig.settings.get()
        previousBackendType = devSettings.backendType
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? =
        if (DevBuild.IS_DEV_BUILD) {
            inflater.inflate(R.layout.dev_settings_screen, container, false)
        } else {
            // We should never reach this code since this fragment should not be visible except in dev builds.
            // Log an error just in case of a bug.
            throw RuntimeException("DevSettings Fragment accessed in non-dev build.")
        }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        dbx_toolbar_layout.dbx_toolbar.title = "Dev Settings"

        stormcrow_override_button
            .setOnClickListener {
                StormcrowOverrideFragment()
                    .show(activity, supportFragmentManager)
            }

        addRadioButtons(dev_settings_vm_radio_button, devSettings.backendNames,
                devSettings.backendNames[devSettings.currentBackendIndex])
        dev_settings_vm_radio_button.setOnCheckedChangeListener(
            RadioGroup.OnCheckedChangeListener { group, checkedId ->
                val radioButton = group.findViewById<RadioButton>(checkedId)
                val checkedBackendType = devSettings.backendConfigs[checkedId].type
                if (!radioButton.isChecked || devSettings.backendType ===
                    checkedBackendType
                ) {
                    // Ignore the case when the existing button is being unchecked
                    // Ignore duplicate onCheckedChanged events being called for buttons which are already checked
                    return@OnCheckedChangeListener
                }
                devSettings.backendType = checkedBackendType
                // Add the user's for the selected VM.
                createUsersRadioButton()
                createTierRadioButtons()

                if (devSettings.backendType == BackendType.DEVBOX) {
                    dev_settings_devbox_owner_group.visibility = View.VISIBLE
                } else {
                    dev_settings_devbox_owner_group.visibility = View.GONE
                }
            })

        if (devSettings.backendType == BackendType.DEVBOX) {
            dev_settings_devbox_owner_group.visibility = View.VISIBLE
        }
        dev_settings_devbox_owner.setText(devSettings.devboxOwnerUsername)
        dev_settings_devbox_owner.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                devSettings.devboxOwnerUsername = s.toString()
            }

            override fun beforeTextChanged(
                s: CharSequence?,
                start: Int,
                count: Int,
                after: Int
            ) {
            }

            override fun onTextChanged(
                s: CharSequence?,
                start: Int,
                before: Int,
                count: Int
            ) {
            }
        })

        // Show various login associated with the Backends. Unlike Backend, we do not pre-check any option.
        createUsersRadioButton()

        dev_settings_user_radio_button.setOnCheckedChangeListener { _, checkedId ->
            if (checkedId >= 0) {
                val user = if (devSettings.allUsers.isNotEmpty() &&
                        checkedId < devSettings.allUsers.size)
                    devSettings.allUsers[checkedId] else null
                devSettings.user = user
            }
        }

        createTierRadioButtons()
        dev_settings_tier_radio_button.setOnCheckedChangeListener { _, checkedId ->
            if (checkedId >= 0) {
                devSettings.selectedServerTierIndex =
                    if (checkedId < devSettings.serverTiers.size) checkedId else 0
            }
        }

        dev_settings_done.setOnClickListener { saveStateAndRelaunchApp() }

        // Mark the Backend that was selected as checked after we have created all the buttons.
        dev_settings_vm_radio_button.check(devSettings.currentBackendIndex)

        dev_settings_version.text = AppInfoUtils.getFullVersionString(activity)

        with(dev_settings_version_override) {
            hint = AppInfoUtils.getAppVersion(activity).toString()
            setText(devSettings.appVersionRequestHeader)
            addTextChangedListener(object : TextWatcher {
                override fun afterTextChanged(s: Editable?) {
                    devSettings.appVersionRequestHeader = s.toString()
                }

                override fun beforeTextChanged(
                    s: CharSequence?,
                    start: Int,
                    count: Int,
                    after: Int
                ) {
                }

                override fun onTextChanged(
                    s: CharSequence?,
                    start: Int,
                    before: Int,
                    count: Int
                ) {
                }
            })
        }
    }

    override fun onBackPressed(): Boolean {
        if (previousBackendType != devSettings.backendType) {
            saveStateAndRelaunchApp()
        }
        return super.onBackPressed()
    }

    private fun addRadioButtons(
        radioGroup: RadioGroup,
        buttonNames: List<String>,
        selectedButton: String?
    ) {
        radioGroup.clearCheck()
        radioGroup.removeAllViews()
        for (i in buttonNames.indices) {
            // Add a radio button for the user.
            val rdbut = RadioButton(context)
            rdbut.id = i
            rdbut.text = buttonNames[i]
            rdbut.setTextColor(Color.WHITE)
            if (rdbut.text == selectedButton) {
                rdbut.isChecked = true
            }
            radioGroup.addView(rdbut)
        }
    }

    fun createUsersRadioButton() =
        addRadioButtons(dev_settings_user_radio_button, devSettings.allUsers, devSettings.user)

    private fun createTierRadioButtons() {
        addRadioButtons(dev_settings_tier_radio_button, devSettings.serverTiers,
            devSettings.serverTier)
        dev_settings_tier_radio_button.check(devSettings.selectedServerTierIndex)
    }

    private fun saveStateAndRelaunchApp() {
        if (!StringUtils.isEmpty(devSettings.appVersionRequestHeader) &&
            AppVersion.fromString(devSettings.appVersionRequestHeader) == null
        ) {
            SnackbarFactory.showSnackbarOrToast(activity,
                "Create a AppVersion from a string of the form \"X.Y.Z\"")
        } else {
            TextProgressDialogFrag.newInstance("Applying changes...")
                .show(activity, supportFragmentManager)
            devSettings.saveState(context!!)
            ProcessPhoenix.triggerRebirth(context!!)
        }
    }
}
