/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.filemanager

import android.net.Uri
import android.os.Bundle
import com.dropbox.core.v2.sharing.DirectLinkMetadata
import com.dropbox.core.v2.sharing.SharedLinkMetadata
import com.dropbox.internalclient.UserApi

/**
 * This class constructs chooser result based on different type of metadata returned by server
 */
class DbxChooserResult(val bundle: Bundle) {

    companion object {
        /* Bundle keys constants */
        val URI = "uri"
        val NAME = "name"
        val ICON_LINK = "icon"
        val ID = "id"
        val PATH_LOWER = "pathLower"
        val PREVIEW_TYPE = "previewType"
        val THUMBNAILS = "thumbnails"
        val SIZE = "bytes"

        /**
         * Factory method using the API v1 server chooser share result
         */
        fun constructFromChooserLink(link: UserApi.ChooserLink): DbxChooserResult =
                DbxChooserResult(bundleFromChooserLink(link))

        private fun bundleFromChooserLink(link: UserApi.ChooserLink) = Bundle().apply {
            putParcelable(URI, Uri.parse(link.link))
            putString(NAME, link.name)
            val thumbnailBundle = Bundle().apply {
                for (thumbInfo in link.thumbnailUrls) {
                    putParcelable(thumbInfo.key, Uri.parse(thumbInfo.value))
                }
            }
            putParcelable(THUMBNAILS, thumbnailBundle)
            putParcelable(ICON_LINK, Uri.parse(link.icon))
            putLong(SIZE, link.bytes)
        }

        /**
         * Factory method using the API v2 server "direct" result.
         */
        fun constructFromDirectLinkMetadata(link: DirectLinkMetadata): DbxChooserResult =
                DbxChooserResult(bundleFromDirectLink(link))

        private fun bundleFromDirectLink(directLinkMetadata: DirectLinkMetadata) = Bundle().apply {
            putParcelable(URI, Uri.parse(directLinkMetadata.url))
            putString(NAME, directLinkMetadata.name)
            val thumbnailBundle = Bundle().apply {
                for ((key, value) in directLinkMetadata.thumbnailUrls) {
                    putParcelable(key, Uri.parse(value))
                }
            }
            putParcelable(THUMBNAILS, thumbnailBundle)
            putString(ICON_LINK, directLinkMetadata.iconUrl)
            putLong(SIZE, directLinkMetadata.size)
        }

        /**
         * Factory method using the API v2 server "preview" result.
         */
        fun constructFromSharedLinkMetadata(link: SharedLinkMetadata): DbxChooserResult =
                DbxChooserResult(bundleFromSharedLink(link))

        private fun bundleFromSharedLink(sharedLinkMetadata: SharedLinkMetadata) = Bundle().apply {
            putParcelable(URI, Uri.parse(sharedLinkMetadata.url))
            putString(NAME, sharedLinkMetadata.name)
            putString(ICON_LINK, sharedLinkMetadata.iconUrl)
            putString(ID, sharedLinkMetadata.id)
            putString(PATH_LOWER, sharedLinkMetadata.pathLower)
            putString(PREVIEW_TYPE, sharedLinkMetadata.previewType)
        }
    }
}
