/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.filemanager

import android.net.Uri
import com.dropbox.android.media.DropboxTranscodeLink
import com.dropbox.android.media.VideoMetadata
import com.dropbox.base.oxygen.Password
import com.dropbox.client2.DropboxAPI
import com.dropbox.client2.DropboxFileInfo
import com.dropbox.client2.ProgressListener
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.stormcrow.Stormcrow
import com.dropbox.core.stormcrow.StormcrowAndroidSharedLinkApiUseListFolder
import com.dropbox.core.v2.DbxClientV2
import com.dropbox.core.v2.files.ListFolderErrorException
import com.dropbox.core.v2.files.SharedLink
import com.dropbox.hairball.entry.SharedLinkEntry
import com.dropbox.internalclient.SharedLinkApi
import com.dropbox.internalclient.UserApi
import com.dropbox.product.dbapp.path.SharedLinkPath
import com.google.common.base.Optional
import java.io.OutputStream

class UserSharedLinkApi(
    private val stormcrow: Stormcrow,
    private val apiV1: UserApi,
    private val apiV2: DbxClientV2
) : SharedLinkApi {
    override fun getRedirectUriForSharedDeepLink(uriPath: String): Uri {
        return apiV1.getRedirectUriForSharedDeepLink(uriPath)
    }

    override fun getSharedLinkMetadata(
        urlPath: String,
        password: Optional<Password>
    ): SharedLinkEntry {
        return apiV1.getSharedLinkMetadata(
            urlPath,
            null,
            password,
            1,
            null,
            false,
            false
        )
    }

    override fun getSharedLinkFileMetadata(
        path: SharedLinkPath,
        password: Optional<Password>,
        includeActivityInfo: Boolean
    ): SharedLinkEntry {
        require(!path.isDir)

        return try {
            apiV1.getSharedLinkMetadata(
                path.urlPath,
                path.relativePath.orNull(),
                password,
                1,
                null,
                false,
                includeActivityInfo
            )
        } catch (exception: DropboxException) {
            throw exception.toSharedLinkApiException()
        }
    }

    override fun getSharedLinkFolderMetadata(
        path: SharedLinkPath,
        password: Optional<Password>,
        hash: String?,
        includeActivityInfo: Boolean
    ): SharedLinkEntry {
        require(path.isDir)

        if (isUsingV2Api()) {
            val sharedLink = SharedLink(path.urlPath, password.orNull()?.passwordString)
            val relativePath = if (path.relativePath.isPresent) path.relativePath.get() else ""

            // TODO(kshimizu): This implementation is still WIP and needs to implement
            // proper pagination

            val listFolderResult = try {
                apiV2.files()
                    .listFolderBuilder(relativePath)
                    .withSharedLink(sharedLink)
                    .start()
            } catch (exception: ListFolderErrorException) {
                throw exception.toSharedLinkApiException()
            }

            return listFolderResult.sharedLinkFolderMetadataSdkToClient(path)
        } else {
            try {
                return apiV1.getSharedLinkMetadata(
                    path.urlPath,
                    path.relativePath.orNull(),
                    password,
                    -1,
                    hash,
                    true,
                    includeActivityInfo
                )
            } catch (exception: DropboxException) {
                throw exception.toSharedLinkApiException()
            }
        }
    }

    override fun getSharedLinkThumbnail(
        path: SharedLinkPath,
        password: Optional<Password>,
        os: OutputStream,
        size: DropboxAPI.ThumbSize,
        format: DropboxAPI.ThumbFormat,
        progressListener: ProgressListener?
    ) {
        apiV1.getSharedLinkThumbnail(path, password, os, size, format, progressListener)
    }

    override fun getSharedLinkFile(
        path: SharedLinkPath,
        password: Optional<Password>,
        os: OutputStream,
        listener: ProgressListener?
    ): DropboxFileInfo {
        return apiV1.getSharedLinkFile(path, password, os, listener)
    }

    override fun getSharedLinkPreview(
        path: SharedLinkPath,
        password: Optional<Password>,
        os: OutputStream,
        listener: ProgressListener?
    ): String {
        return apiV1.getSharedLinkPreview(path, password, os, listener)
    }

    override fun getSharedLinkMedia(
        path: SharedLinkPath,
        password: Optional<Password>
    ): DropboxAPI.DropboxLink {
        return apiV1.getSharedLinkMedia(path, password)
    }

    override fun getSharedLinkMediaTranscode(
        path: SharedLinkPath,
        password: Optional<Password>,
        screenResolution: String,
        connectionType: String,
        appVersion: String
    ): DropboxTranscodeLink {
        return apiV1.getSharedLinkMediaTranscode(
            path, password, screenResolution, connectionType, appVersion
        )
    }

    override fun getSharedLinkFileUri(path: SharedLinkPath): String {
        return apiV1.getSharedLinkFileUri(path)
    }

    override fun fetchVideoTranscodeProgress(link: String): Int {
        return apiV1.fetchVideoTranscodeProgress(link)
    }

    override fun fetchVideoMetadata(link: String): VideoMetadata {
        return apiV1.fetchVideoMetadata(link)
    }

    override fun getUserId(): Optional<String> {
        return apiV1.getUserId()
    }

    private fun isUsingV2Api() =
        try {
            stormcrow.isInVariantUnlogged(StormcrowAndroidSharedLinkApiUseListFolder.VENABLED)
        } catch (e: com.dropbox.base.error.DbxException) {
            false
        }
}
