/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.filemanager

import com.dropbox.client2.DropboxAPI
import com.dropbox.client2.RESTUtility
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.legacy_api.exception.DropboxIOException
import com.dropbox.core.legacy_api.exception.DropboxParseException
import com.dropbox.core.legacy_api.exception.DropboxServerException
import com.dropbox.core.legacy_api.exception.DropboxUnlinkedException
import com.dropbox.core.v2.files.FileMetadata
import com.dropbox.core.v2.files.FileMetadataUtils
import com.dropbox.core.v2.files.FolderMetadata
import com.dropbox.core.v2.files.ListFolderErrorException
import com.dropbox.core.v2.files.ListFolderResult
import com.dropbox.core.v2.files.Metadata
import com.dropbox.hairball.entry.CloudDocClass
import com.dropbox.hairball.entry.SharedLinkEntry
import com.dropbox.internalclient.SharedLinkApiException
import com.dropbox.product.dbapp.path.SharedLinkPath

fun ListFolderResult.sharedLinkFolderMetadataSdkToClient(path: SharedLinkPath): SharedLinkEntry =
    sharedLinkEntryFromTopLevelFolderMetadata(
        path,
        folderMetadataSdkToClient(
            entries,
            path.getRelativePathOrDefault()
        )
    )

fun DropboxException.toSharedLinkApiException(): SharedLinkApiException {
    var errorCode = ""
    val type = when (this) {
        is DropboxIOException -> SharedLinkApiException.Type.IO
        is DropboxParseException -> SharedLinkApiException.Type.PARSE
        // SharedLinkMetadataManager wants to consider DropboxUnlinkedException as if the content
        // has not been found, so we make the translation here to NOT_FOUND
        is DropboxUnlinkedException -> SharedLinkApiException.Type.NOT_FOUND
        is DropboxServerException ->
            when (error) {
                DropboxServerException._304_NOT_MODIFIED ->
                    SharedLinkApiException.Type.NOT_MODIFIED
                DropboxServerException._403_FORBIDDEN ->
                    SharedLinkApiException.Type.FORBIDDEN
                DropboxServerException._404_NOT_FOUND ->
                    SharedLinkApiException.Type.NOT_FOUND
                in 500..599 -> SharedLinkApiException.Type.SERVER
                else -> {
                    errorCode = error.toString()
                    SharedLinkApiException.Type.GENERIC
                }
            }
        else -> SharedLinkApiException.Type.GENERIC
    }

    return SharedLinkApiException(errorCode, type)
}

fun ListFolderErrorException.toSharedLinkApiException(): SharedLinkApiException =
    if (errorValue.isPath && errorValue.pathValue.isNotFound) {
        SharedLinkApiException("", SharedLinkApiException.Type.NOT_FOUND)
    } else {
        SharedLinkApiException("", SharedLinkApiException.Type.GENERIC)
    }

private fun sharedLinkEntryFromFileMetadata(
    metadata: FileMetadata,
    relativePath: String
): SharedLinkEntry = sharedLinkEntryBuilder(
    icon = metadata.icon,
    isDir = false,
    // FIXME
    // Path is being constructed here because the endpoint is returning null for display_path
    // and lower_path. This will result in canonicalization issues. Path is returning null
    // because the content is unmounted. We need to figure out what needs to be done to be
    // able to return the display_path so we can use it, the substitute that for the constructed
    // path.
    path = relativePath + metadata.name,
    bytes = metadata.size,
    modified = RESTUtility.formatDate(metadata.serverModified),
    clientMTime = RESTUtility.formatDate(metadata.clientModified),
    mimeType = metadata.mimeType,
    rev = metadata.rev,
    thumbExists = FileMetadataUtils.guessThumbExists(metadata),
    cloudDocClass = FileMetadataUtils.getCloudDocClass(metadata)
)

private fun sharedLinkEntryFromFolderMetadata(
    metadata: FolderMetadata,
    relativePath: String
): SharedLinkEntry {
    var noAccess = metadata.sharingInfo?.noAccess ?: false

    return sharedLinkEntryBuilder(
        icon = "folder",
        isDir = true,
        // FIXME
        // Path is being constructed here because the endpoint is returning null for display_path
        // and lower_path. This will result in canonicalization issues. Path is returning null
        // because the content is unmounted. We need to figure out what needs to be done to be
        // able to return the display_path so we can use it, the substitute that for the constructed
        // path.
        path = relativePath + metadata.name,
        folderName = metadata.name,
        noAccess = noAccess
    )
}

private fun sharedLinkEntryFromTopLevelFolderMetadata(
    path: SharedLinkPath,
    contents: List<SharedLinkEntry>
): SharedLinkEntry {
    // We no longer use the hash parameter in ApiV2, but are forced to pass it for now in order
    // to bypass an assertion in legacy code.
    val fakeHash = "fakehash"

    return sharedLinkEntryBuilder(
        icon = "folder",
        isDir = true,
        path = path.getRelativePathOrDefault("/"),
        folderName = path.name,
        hash = fakeHash,
        visibility = DropboxAPI.SharedLinkVisibility.PUBLIC.name,
        contents = contents
    )
}

private fun sharedLinkEntryBuilder(
    icon: String,
    isDir: Boolean,
    path: String,
    folderName: String? = null,
    bytes: Long = 0,
    hash: String? = null,
    modified: String? = null,
    clientMTime: String? = null,
    mimeType: String? = null,
    rev: String? = null,
    thumbExists: Boolean = false,
    visibility: String? = null,
    contents: List<SharedLinkEntry>? = null,
    noAccess: Boolean = false,
    cloudDocClass: CloudDocClass = CloudDocClass.NONE
): SharedLinkEntry {
    //
    // WARNING: `FolderMetadata` and `FileMetadata` do not have enough information to correctly
    // create SharedLinkEntries. We are attempting to provide sensible defaults, but there is
    // work to be done around deprecating unused fields, and getting the endpoint to pass back
    // all the data that we need to create actual objects.
    //

    val permissions = DropboxAPI.SharedLinkPermissions(true, false)

    return SharedLinkEntry(
        false,
        folderName,
        bytes,
        hash,
        icon,
        isDir,
        modified,
        clientMTime,
        path,
        mimeType,
        rev,
        thumbExists,
        visibility,
        null,
        permissions,
        contents,
        null,
        noAccess,
        cloudDocClass
    )
}

private fun folderMetadataSdkToClient(
    metadataList: List<Metadata>,
    relPath: String
): List<SharedLinkEntry> =
    metadataList
        .map { extractSharedLinkEntry(it, relPath + "/") }
        .filterNotNull()

private fun extractSharedLinkEntry(
    metadata: Metadata,
    relativePath: String
): SharedLinkEntry? =
    when (metadata) {
        is FolderMetadata -> sharedLinkEntryFromFolderMetadata(metadata, relativePath)
        is FileMetadata -> sharedLinkEntryFromFileMetadata(metadata, relativePath)
        else -> null
    }

private fun SharedLinkPath.getRelativePathOrDefault(default: String = ""): String =
    if (relativePath.isPresent) relativePath.get() else default
