package com.dropbox.android.filemanager.downloading

import com.dropbox.android.taskqueue.convertToThumbnailFormat
import com.dropbox.android.taskqueue.convertToThumbnailMode
import com.dropbox.android.taskqueue.convertToThumbnailSize
import com.dropbox.client2.DropboxAPI.ThumbFormat
import com.dropbox.client2.DropboxAPI.ThumbSize
import com.dropbox.client2.DropboxFileInfo
import com.dropbox.client2.ProgressListener
import com.dropbox.core.DbxException
import com.dropbox.core.legacy_api.exception.DropboxException
import com.dropbox.core.stormcrow.Stormcrow
import com.dropbox.core.stormcrow.StormcrowAndroidUseDbxFilesFetchApiV2
import com.dropbox.core.v2.DbxClientV2
import com.dropbox.hairball.entry.DropboxEntry
import com.dropbox.internalclient.UserApi
import com.dropbox.product.dbapp.path.DropboxPath
import java.io.IOException
import java.io.OutputStream

/**
 * Implementation of [FileDownloadService] for [DropboxPath]s.
 */
class DropboxFileDownloadService(
    private val api: UserApi,
    private val apiv2: DbxClientV2,
    private val stormcrow: Stormcrow
) : FileDownloadService<DropboxPath> {

    /**
     * Get the requested file information from server.
     * @param path the Dropbox path to the file.
     * @param revision the revision (from the file's metadata) of the file to download
     * @param os the [OutputStream] to write the file to.
     * @param listener an optional [ProgressListener] to receive progress
     * updates as the file downloads, or null.
     * @return the [DropboxFileInfo] for the downloaded file.
     * @throws DbxException The base exception thrown by Dropbox API calls
     * @throws IOException for some network-related errors.
     * @throws DropboxException for any other unknown errors. This is also a
     * superclass of all other Dropbox exceptions
     */
    @Throws(DbxException::class, IOException::class, DropboxException::class)
    override fun getFileFromServer(
        path: DropboxPath,
        revision: String,
        os: OutputStream,
        listener: ProgressListener?
    ): DropboxFileInfo {

        return if (fetchApiV2FilesEnabled()) {
            val loader = apiv2.files().download(path.asCanonicalPath(), revision)
            val fileMetadata = loader.download(os)
            DropboxFileInfo(
                fileMetadata.mimeType,
                fileMetadata.size,
                DropboxEntry(fileMetadata)
            )
        } else {
            api.getFile(path, revision, os, listener)
        }
    }

    /**
     * Get the requested file thumbnail information from server and write to output stream
     * @param path the Dropbox path to the file.
     * @param rev the revision (from the file's metadata) of the thumbnail to download
     * @param os the [OutputStream] to write the thumbnail to.
     * @param size the size of thumbnail
     * @param format the format of thumbnail
     * @param progressListener an optional [ProgressListener] to receive progress updates
     * @throws DbxException The base exception thrown by Dropbox API calls
     * @throws IOException for some network-related errors.
     * @throws DropboxException for any other unknown errors. This is also a
     * superclass of all other Dropbox exceptions
     */
    @Throws(DbxException::class, IOException::class, DropboxException::class)
    override fun getThumbnailFromServer(
        path: DropboxPath,
        rev: String,
        os: OutputStream,
        size: ThumbSize,
        format: ThumbFormat,
        progressListener: ProgressListener?
    ) {
        if (fetchApiV2FilesEnabled()) {
            val loader = apiv2.files()
                .getThumbnailBuilder(path.asCanonicalPath())
                .withFormat(format.convertToThumbnailFormat())
                .withSize(size.convertToThumbnailSize())
                .withMode(size.convertToThumbnailMode())
                .start()
            loader.download(os)
        } else {
            api.getThumbnail(path, rev, os, size, format, progressListener)
        }
    }

    /**
     * Get the requested preview of the file, write to output stream and return the MIME type
     * @param path the Dropbox path to the file.
     * @param rev the revision (from the file's metadata) of the preview to download
     * @param os the [OutputStream] to write the thumbnail to.
     * @param listener a required [ProgressListener] to receive progress updates
     * @return the MIME type for the preview
     * @throws DbxException The base exception thrown by Dropbox API calls
     * @throws IOException for some network-related errors.
     * @throws DropboxException for any other unknown errors. This is also a superclass of all other Dropbox exceptions
     */
    @Throws(DbxException::class, DropboxException::class, IOException::class)
    override fun getPreviewFromServer(
        path: DropboxPath,
        rev: String,
        os: OutputStream,
        listener: ProgressListener
    ): String {
        return if (fetchApiV2FilesEnabled()) {
            val loader = apiv2.files().getPreview(path.asCanonicalPath(), rev)
            loader.download(os)
            // As a side effect, we also get contentType from loader to tell which type of file we are previewing with
            loader.contentType
        } else {
            api.getPreview(path, rev, os, listener)
        }
    }

    private fun fetchApiV2FilesEnabled(): Boolean {
        return try {
            stormcrow.isInVariantLogged(StormcrowAndroidUseDbxFilesFetchApiV2.VAPIV2)
        } catch (e: DbxException) {
            false
        }
    }
}
