/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.notifications

import android.annotation.TargetApi
import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.support.annotation.StringRes
import com.dropbox.android.R
import com.dropbox.base.device.Version
import com.dropbox.base.device.Version.API_26

enum class ChannelType(
    @StringRes private val idRes: Int,
    @StringRes private val nameRes: Int,
    private val importance: Importance
) {

    DEFAULT(
            R.string.channel_default_id,
            R.string.channel_default_name,
            Importance.DEFAULT),
    CAMERA_UPLOADS(
            R.string.channel_camera_upload_id,
            R.string.channel_camera_upload_name,
            Importance.DEFAULT),
    FILE_TRANSFER(
            R.string.channel_file_transfer_id,
            R.string.channel_file_transfer_name,
            Importance.LOW)
    ;

    fun getId(context: Context): String = context.resources.getString(idRes)
    internal fun getName(context: Context): String = context.resources.getString(nameRes)
    internal fun getImportance() = importance.value

    /** This is a little awkward, but this structure removes the api version check from the
     * "master" list of channel types so that it should be easier to skim the configuration settings
     * for each type */
    private enum class Importance {
        DEFAULT() {
            override val value: Int
                get() = if (Version.gte(API_26)) {
                    NotificationManager.IMPORTANCE_DEFAULT
                } else {
                    0
                }
        },
        LOW() {
            override val value: Int
                get() = if (Version.gte(API_26)) {
                    NotificationManager.IMPORTANCE_LOW
                } else {
                    0
                }
        }
        ;

        abstract val value: Int
    }
}

/**
 * Helper class to set notification channels, but only on API 26+ because the NotificationChannel
 * class is new and not in the support library
 */
internal class NotificationChannelHelper(val context: Context) {

    private var manager: NotificationManager = context.getSystemService(
            Context.NOTIFICATION_SERVICE) as NotificationManager

    /**
     * Initialize the channel helper by setting up channels
     */
    fun initChannels() {
        setupChannels()
    }

    /**
     * Recreate the channels if locale changes
     */
    fun setupLocale() = setupChannels()

    /**
     * Create all channels defined in {@link ChannelType}
     */
    @TargetApi(API_26)
    private fun setupChannels() {
        if (Version.gte(API_26)) {
            for (type in ChannelType.values()) {
                createNotificationChannel(type)
            }
        }
    }

    @TargetApi(API_26)
    private fun createNotificationChannel(type: ChannelType) {
        // Also allow further customize channel's default notification behaviors, customization
        // should happen before creating the channel
        val channel = NotificationChannel(
                type.getId(context),
                type.getName(context),
                type.getImportance())

        // Register the channel with the system; you can't change the importance
        // or other notification behaviors after this
        manager.createNotificationChannel(channel)
    }
}
