// dbx-min-coverage: 0% because fragment
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.preference

import android.os.Bundle
import android.support.design.widget.Snackbar
import android.support.v7.preference.Preference
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.dropbox.android.DropboxApplication
import com.dropbox.android.R
import com.dropbox.android.activity.SnackbarProvider
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.activity.base.BaseUserPreferenceFragment
import com.dropbox.android.settings.UserProperties
import com.dropbox.android.user.DbxUser.UserRole
import com.dropbox.android.user.UserSelector
import com.dropbox.android.util.SnackbarHelper
import com.dropbox.base.device.DeviceFeatures
import com.google.common.base.Preconditions

/**
 * [android.support.v7.preference.PreferenceFragmentCompat] that contains settings related to
 * camera uploads.
 */
class CameraUploadPreferenceFragment : BaseUserPreferenceFragment(), SnackbarProvider {

    companion object {

        /** Returns a new instance of [CameraUploadPreferenceFragment]  */
        fun newInstance(role: UserRole): CameraUploadPreferenceFragment =
            CameraUploadPreferenceFragment().apply {
                addUserSelectorToArgs(UserSelector.forRole(role))
            }
    }

    private lateinit var deviceFeatures: DeviceFeatures
    private lateinit var properties: UserProperties
    private lateinit var controller: CameraUploadPreferenceFragmentController

    private val snackbarHelper = SnackbarHelper()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        val user = user
        val activity = checkNotNull(activity)

        deviceFeatures = DeviceFeatures(activity.applicationContext)
        properties = user.userProperties

        controller = CameraUploadPreferenceFragmentController(
            fragment = this,
            user = user,
            cameraUploadsManager = DropboxApplication.getCameraUploadsManager(activity),
            properties = properties
        )

        // Load the preferences from an XML resource
        addPreferencesFromResource(R.xml.camera_upload_preferences)
        setupPreferences()
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        val view = checkNotNull(super.onCreateView(inflater, container, savedInstanceState))
        snackbarHelper.init(view)
        // set the content description so that we can locate this view in ui tests.
        mRecyclerView.contentDescription = resources.getString(R.string.preferences_fragment_list)
        return view
    }

    override fun onResume() {
        super.onResume()

        (activity as PreferenceActivity).apply {
            setTitle(R.string.settings_camera_upload_section_title)
            setupHeroHeaderUsing(mRecyclerView)
        }
    }

    private fun setupPreferences() {
        val cameraUploadSwitch = getPreference(SafePreference.CAMERA_UPLOAD_SWITCH)
        val cameraUploadLimitVideoUploadsOverCell =
            getPreference(SafePreference.CAMERA_UPLOAD_3G_LIMIT)
        val cameraUploadUseCellularData =
            getPreference(SafePreference.CAMERA_UPLOAD_USE_CELLULAR_DATA)
        val cameraUploadUploadVideo = getPreference(SafePreference.CAMERA_UPLOAD_UPLOAD_VIDEO)
        val cameraUploadDisableThreshold =
            getPreference(SafePreference.CAMERA_UPLOAD_DISABLE_BATTERY_LEVEL)

        cameraUploadSwitch.setOnPreferenceChangeListener { _, _ ->
            val activity = activity as BaseActivity? ?: return@setOnPreferenceChangeListener true

            controller.toggleCu(activity)
            false
        }

        cameraUploadDisableThreshold.setOnPreferenceChangeListener { _, newValue ->
            controller.setCuBatteryThreshold(newValue as Int)
            true
        }
        cameraUploadUseCellularData.setOnPreferenceChangeListener { _, newValue ->
            activity as BaseActivity? ?: return@setOnPreferenceChangeListener true

            controller.setUse3G(newValue as Boolean)
            true
        }

        cameraUploadUploadVideo.setOnPreferenceChangeListener { _, newValue ->
            activity as BaseActivity? ?: return@setOnPreferenceChangeListener true

            controller.setEnableVideosUpload(newValue as Boolean)
            true
        }
        cameraUploadLimitVideoUploadsOverCell.setOnPreferenceChangeListener { _, newValue ->
            activity as BaseActivity? ?: return@setOnPreferenceChangeListener true

            controller.setLimitVideoUploadsOverCell(newValue == "limit")
            true
        }

        refreshPreferences()
    }

    internal fun refreshPreferences() {
        val cameraUploadSwitch = getPreference(SafePreference.CAMERA_UPLOAD_SWITCH)
        val cameraUploadLimitVideoUploadsOverCell =
            getPreferenceOrNull(SafePreference.CAMERA_UPLOAD_3G_LIMIT)
        val cameraUploadUseCellularData =
            getPreferenceOrNull(SafePreference.CAMERA_UPLOAD_USE_CELLULAR_DATA)
        val cameraUploadUploadVideo = getPreference(SafePreference.CAMERA_UPLOAD_UPLOAD_VIDEO)

        cameraUploadSwitch.isChecked = properties.cameraUploadEnabled()

        if (deviceFeatures.hasMobileData()) {
            cameraUploadUseCellularData.isChecked = properties.cameraUploadUse3G()

            val limit = if (properties.cameraUploadLimitVideoUploadsOverCell()) {
                "limit"
            } else {
                "nolimit"
            }
            cameraUploadLimitVideoUploadsOverCell.value = limit
        } else {
            // Don't show any mobile network-related prefs for Wi-Fi only devices.
            removePreference(cameraUploadUseCellularData)
            removePreference(cameraUploadLimitVideoUploadsOverCell)
        }

        cameraUploadUploadVideo.isChecked = properties.cameraUploadVideosEnabled()

        syncCuDisableThreshold()
    }

    private fun syncCuDisableThreshold() {
        val whenCharging = properties.cameraUploadOnlyWhenCharging()

        val cameraUploadDisableThreshold =
            getPreference(SafePreference.CAMERA_UPLOAD_DISABLE_BATTERY_LEVEL)
        val batteryThresholdString = getString(
            R.string.camera_upload_prefs_battery_level_threshold_desc,
            properties.cameraUploadBatteryThreshold().toString()
        )
        cameraUploadDisableThreshold.summary = if (whenCharging) {
            getString(
                R.string
                    .camera_upload_prefs_battery_level_threshold_only_while_charging
            )
        } else {
            batteryThresholdString
        }
        cameraUploadDisableThreshold.sliderValue =
                properties.cameraUploadBatteryThreshold().toInt()
        cameraUploadDisableThreshold.isOnlyWhenCharging = whenCharging
    }

    override fun onDisplayPreferenceDialog(preference: Preference) {
        Preconditions.checkNotNull(preference)

        if (preference is BatteryLevelThresholdSeekBarDialogPreference) {
            val fragment =
                BatteryLevelThresholdSeekBarPreferenceDialogFragment.newInstance(preference)
            fragment.setTargetFragment(this, 0)
            fragment.show(fragmentManager, "BatteryLevelThresholdSeekBarDialogPreference")
        } else {
            super.onDisplayPreferenceDialog(preference)
        }
    }

    override fun getSnackbarContainer(): View? {
        return snackbarHelper.container
    }

    override fun showSnackbar(snackbar: Snackbar?) {
        snackbarHelper.show(snackbar)
    }

    override fun dismissSnackbar() {
        snackbarHelper.dismiss()
    }
}
