/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.preference

import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.camerauploads.CameraUploadsManager
import com.dropbox.android.settings.UserProperties
import com.dropbox.android.user.DbxUser
import com.dropbox.base.analytics.Analytics
import com.dropbox.base.analytics.CameraUploadsFeatureEvents

/**
 * Controller for [CameraUploadPreferenceFragment] that contains most of the business logic. Allows
 * for easier testing.
 */
class CameraUploadPreferenceFragmentController (
    private val fragment: CameraUploadPreferenceFragment,
    private val user: DbxUser,
    private val cameraUploadsManager: CameraUploadsManager,
    private val properties: UserProperties
) {
    /**
     * Toggles CU, switching it on if it's currently off or vise versa. It will also show
     * confirmation UI if needed.
     */
    fun toggleCu(activity: BaseActivity) {
        if (properties.cameraUploadEnabled()) {
            // Turn it off.
            cameraUploadsManager.disableCameraUploadsForUser(user)

            Analytics.PREF_CHANGED()
                .set("cameraupload.enabled", properties.cameraUploadEnabled())
                .logTo(user.logger)

            fragment.refreshPreferences()
        } else {
            /*
             * User CU eligibility is determined by whether they have the CU
             * capability available or desktop linked.
             */
            cameraUploadsManager.enableCameraUploadsOrShowGatingUI(
                user,
                activity,
                CameraUploadsFeatureEvents.CameraUploadsEnableSource.ACCOUNT_SETTINGS,
                fragment
            ) {
                fragment.refreshPreferences()

                Analytics.PREF_CHANGED()
                    .set("cameraupload.enabled", properties.cameraUploadEnabled())
                    .logTo(user.logger)
            }
        }
    }

    /**
     * Method allowing to set an arbitrary threshold battery level for turning CU off.
     *
     * @param value number from range [BatteryLevelThresholdSeekBarDialogPreference.MINIMUM_VALUE] -
     * [BatteryLevelThresholdSeekBarDialogPreference.MAXIMUM_VALUE] indicating the threshold battery
     * level that will trigger turning CU off or
     * [BatteryLevelThresholdSeekBarPreferenceDialogFragment.ONLY_WHEN_CHARGING]
     */
    fun setCuBatteryThreshold(value: Int) {
        val onlyWhenCharging = value == BatteryLevelThresholdSeekBarPreferenceDialogFragment
            .ONLY_WHEN_CHARGING
        cameraUploadsManager.setEnabledOnlyWhenCharging(user, onlyWhenCharging)
        if (!onlyWhenCharging) {
            cameraUploadsManager.setBatteryThreshold(user, value.toLong())
            Analytics.PREF_CHANGED()
                .set("cameraupload.batterythreshold", value.toLong())
                .logTo(user.logger)
            Analytics.PREF_CHANGED()
                .set("cameraupload.onlywhencharging", false)
                .logTo(user.logger)
        } else {
            Analytics.PREF_CHANGED()
                .set("cameraupload.onlywhencharging", true)
                .logTo(user.logger)
        }
        // After settings have changed, we need to review whether to pause/resume CUs
        user.uploadQueue.bumpScheduledTasks()

        fragment.refreshPreferences()
    }

    /**
     * Sets whether or not we should upload on cell data.
     */
    fun setUse3G(use3G: Boolean) {
        cameraUploadsManager.setUse3G(user, use3G)
        Analytics.PREF_CHANGED()
            .set("cameraupload.usedataplan", use3G)
            .logTo(user.logger)
    }

    /**
     * Enables or disables video upload.
     */
    fun setEnableVideosUpload(enableVideosUpload: Boolean) {
        cameraUploadsManager.setVideosEnabled(user, enableVideosUpload)
        Analytics.PREF_CHANGED()
            .set("cameraupload.enablevideouploads", enableVideosUpload)
            .logTo(user.logger)
    }

    fun setLimitVideoUploadsOverCell(limitVideoUploadsOverCell: Boolean) {
        cameraUploadsManager.setLimitVideoUploadsOverCell(user, limitVideoUploadsOverCell)
        Analytics.PREF_CHANGED()
            .set("cameraupload.limitdataplanfilesize", limitVideoUploadsOverCell)
            .logTo(user.logger)
    }
}
