// dbx-min-coverage: 71% because we need to roll this our to mitigate a SEV
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("ContentUriBlocking")

package com.dropbox.android.provider

import com.dropbox.base.error.DbxException
import com.dropbox.core.stormcrow.NoauthStormcrow
import com.dropbox.core.stormcrow.StormcrowMobileAndroidEnforceContentUriBlacklists

/**
 * Map of packages (and an associated fixed version code) that are write blacklisted. This means
 * that we should block all writes from that package below the version where it was fixes.
 * ({@see #shouldBlockContentUriWrites}).
 *
 * 1.
 * T188481: Adobe Reader is not following the content:// uri protocol, so all ContentURI
 * writes should be blocked for that package below 208062 (version where it was fixed).
 */
private val BLACKLISTED_PACKAGES = mapOf("com.adobe.reader" to 208062)
/**
 * Whether or not [android.content.ContentProvider] implementations (e.g.
 * [com.dropbox.android.provider.FileCacheProvider]) should block any kind of write
 * operations on our data (e.g. [android.content.ContentProvider.openFile] from the
 * given package and version.
 *
  This is useful when for certain packages that are known not to adhered to the content:// uri
 * protocol and inadvertently corrupt or overwrite our data.
 *
 * @param packageName name of the package
 * @param versionCode version code of the package
 *
 */
fun shouldBlockContentUriWritesFor(packageName: String, versionCode: Int) =
    BLACKLISTED_PACKAGES[packageName]?.let { versionCode < it } ?: false

/**
 * Whether or not we should enforce blocking of blacklisted packages
 * (@see[#shouldBlockContentUriWritesFor]). It is mostly a kill switch to allows us to turn it off.
 */
fun shouldEnforceContentUriBlocking(deviceStormcrow: NoauthStormcrow): Boolean {
    return try {
        deviceStormcrow.isInNoauthVariantUnlogged(
                StormcrowMobileAndroidEnforceContentUriBlacklists.VENABLED)
    } catch (e: DbxException) {
        false
    }
}
