/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
package com.dropbox.android.sharing.link

import com.dropbox.android.sharing.SharedContentLinkError
import com.dropbox.android.sharing.SharedLinkInfo
import com.dropbox.android.sharing.SharedLinkLoadResult
import com.dropbox.android.sharing.ShmodelError
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.base.analytics.SharedLinkEvents
import com.dropbox.base.analytics.SharedLinkEvents.LinkAction
import com.dropbox.base.analytics.SharedLinkEvents.PathType

object SharedLinkAnalyticsHelper {
    /**
     * Log the result of a resolving a shared link into an action for the client to take.
     */
    fun logResult(data: SharedLinkLoadResult, logger: AnalyticsLogger) {
        val (action, isDir) = getLogInfo(data)

        val pathType = when (isDir) {
            true -> PathType.FOLDER
            false -> PathType.FILE
            null -> PathType.UNKNOWN
        }

        SharedLinkEvents.ResolveLink()
            .setAction(action)
            .setPathType(pathType)
            .logTo(logger)
    }

    private fun getLogInfo(data: SharedLinkLoadResult) = when {
        data.sharedLinkInfo.isPresent -> {
            val sharedLinkInfo: SharedLinkInfo = data.sharedLinkInfo.get()
            val entry = sharedLinkInfo.entry
            val action = entry.getAsSuccessAction()
            val logAction = when (action) {
                SharedLinkSuccessAction.REDIRECT_TO_BROWSE_FOLDER ->
                    LinkAction.REDIRECT_TO_BROWSE_FOLDER
                SharedLinkSuccessAction.REDIRECT_TO_BROWSE_FILE ->
                    LinkAction.REDIRECT_TO_BROWSE_FILE
                SharedLinkSuccessAction.PREVIEW_FOLDER -> LinkAction.PREVIEW_FOLDER
                SharedLinkSuccessAction.MOUNT_OR_PREVIEW_FOLDER ->
                    LinkAction.MOUNT_OR_PREVIEW_FOLDER
                SharedLinkSuccessAction.PREVIEW_FILE -> LinkAction.PREVIEW_FILE
            }
            logAction to null
        }
        data.shmodelError.isPresent -> {
            val error: ShmodelError = data.shmodelError.get()
            val action = when (error) {
                ShmodelError.EXPIRED -> LinkAction.LINK_EXPIRED
                ShmodelError.FORBIDDEN -> LinkAction.LINK_FORBIDDEN
                ShmodelError.INVALID -> LinkAction.LINK_INVALID
                ShmodelError.NONEXISTENT -> LinkAction.LINK_NONEXISTENT
                ShmodelError.UNSUPPORTED -> LinkAction.LINK_UNSUPPORTED
                ShmodelError.DISABLED -> LinkAction.LINK_DISABLED
                ShmodelError.TAKEDOWN -> LinkAction.LINK_TAKEDOWN
                ShmodelError.TOO_MANY_ENTRIES -> LinkAction.LINK_TOO_MANY_ENTRIES
                ShmodelError.PASSWORD_ERROR -> LinkAction.LINK_PASSWORD_ERROR
                ShmodelError.NETWORK_ERROR -> LinkAction.NETWORK_ERROR
                ShmodelError.UNKNOWN -> LinkAction.UNKNOWN
            }
            action to null
        }
        data.sharedContentLinkError.isPresent -> {
            val error: SharedContentLinkError = data.sharedContentLinkError.get()
            when (error) {
                is SharedContentLinkError.RedirectToClaiming ->
                    LinkAction.REDIRECT_TO_CLAIM to error.contentIsDir
                is SharedContentLinkError.AccountSwitch ->
                    LinkAction.ACCOUNT_SWITCH to error.contentIsDir
                is SharedContentLinkError.RequiredEmailVerify ->
                    LinkAction.REQUIRED_EMAIL_VERIFY to error.contentIsDir
                is SharedContentLinkError.OverQuota -> LinkAction.OVER_QUOTA to error.isDir
                is SharedContentLinkError.RedirectToRequestAccess ->
                    LinkAction.REDIRECT_TO_REQUEST_ACCESS to error.contentIsDir
                is SharedContentLinkError.RequireSync -> LinkAction.REQUIRE_SYNC to null
                is SharedContentLinkError.RequirePassword -> LinkAction.REQUIRE_PASSWORD to null
                is SharedContentLinkError.RedirectToTeamJoin ->
                    LinkAction.REDIRECT_TO_TEAM_JOIN to error.contentIsDir
                is SharedContentLinkError.RedirectToSignIn -> LinkAction.REDIRECT_TO_SIGN_IN to null
                is SharedContentLinkError.DenyAccess -> LinkAction.DENY_ACCESS to null
                is SharedContentLinkError.Network -> LinkAction.NETWORK_ERROR to null
                is SharedContentLinkError.Unknown -> LinkAction.UNKNOWN to null
                else -> LinkAction.UNKNOWN to null
            }
        }
        else -> {
            // The SharedLinkLoadResult constructor guarantees this won't happen
            throw IllegalArgumentException("Unknown SharedLinkLoadResult")
        }
    }
}
