/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.taskqueue

import android.content.Context
import android.os.AsyncTask
import com.dropbox.android.R
import com.dropbox.android.filemanager.downloading.DownloadManager
import com.dropbox.android.filemanager.status.ExportingStatus
import com.dropbox.android.util.PathHelper
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.hairball.metadata.NetworkException
import com.dropbox.hairball.taskqueue.TaskResult.ResultType
import com.dropbox.hairball.taskqueue.TaskResult.ResultType.SUCCESS
import com.dropbox.hairball.taskqueue.TaskResult.ResultType.CANCELED
import com.dropbox.hairball.taskqueue.TaskResult.ResultType.NETWORK_ERROR
import com.dropbox.hairball.taskqueue.TaskResult.ResultType.PERM_NETWORK_ERROR
import com.dropbox.hairball.taskqueue.TaskResult.ResultType.DUPLICATE_DOWNLOAD
import com.dropbox.product.dbapp.path.Path

/**
 * Abstract class for export task which kicks off a blocking download of the path to be exported
 * and then passes the result to the subclass to handle the rest.
 */
abstract class AbstractExportTask<T : Path>(
    protected val context: Context,
    @Suppress("DEPRECATION") protected val toaster: com.dropbox.android.util.BackgroundToaster,
    protected val pathHelper: PathHelper<T>,
    protected val logger: AnalyticsLogger
)
    : AsyncTask<Any, Void, ResultType>() {

    protected abstract fun handleDownloadResult(
        downloadResult: DownloadManager.Result<T>
    ): ResultType

    final override fun doInBackground(vararg param: Any): ResultType {
        val metadataService = pathHelper.metadataService
        val downloadManager = pathHelper.downloadManager ?: return ResultType.FAILURE

        var entry = metadataService.getEntryOrNull(getPath())

        // It could be the case that we don't have cached metadata for a given path (e.g. paths that
        // we retrieved from Search). So let's make an attempt to cache it.
        if (entry == null) {
            try {
                entry = metadataService.getLocalEntryFromServer(getPath())
            } catch (e: NetworkException) {
                return ResultType.NETWORK_ERROR
            } catch (e: Exception) {
                return ResultType.FAILURE
            }
        }

        val downloadRequest = DownloadManager.Request(entry).setDownloadingStatus(
                ExportingStatus::class.java)

        val downloadResult = downloadManager.blockingDownload(
                downloadRequest,
                object : DownloadManager.BlockingDownloadListener {
                    override fun onStart(initialDownloadInfo: DownloadManager.InitialDownloadInfo) {
                        // Do nothing
                    }

                    override fun onProgress(progress: Long, total: Long) {
                        // Do nothing
                    }
                })

        return handleDownloadResult(downloadResult)
    }

    final override fun onPostExecute(status: ResultType) {
        when (status) {
            SUCCESS -> toaster.displayToast(R.string.export_successful, getPath().name)
            CANCELED -> { /*Do nothing*/ }
            NETWORK_ERROR, PERM_NETWORK_ERROR -> toaster.displayToast(
                    R.string.export_network_error, getPath().name)
            DUPLICATE_DOWNLOAD -> toaster.displayToast(R.string.task_status_duplicate_download)
            else -> toaster.displayToast(R.string.export_error, getPath().name)
        }
    }

    fun getPath(): T {
        return pathHelper.path
    }
}
