// dbx-min-coverage: 33% because UI

/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */

package com.dropbox.android.update

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.util.Pair
import android.view.View
import com.dropbox.android.DropboxApplication
import com.dropbox.android.R
import com.dropbox.android.activity.DropboxBrowser
import com.dropbox.android.activity.base.BaseActivity
import com.dropbox.android.util.EmmHelper
import com.dropbox.android.util.InternalBuild
import com.dropbox.android.widget.FullscreenImageTitleTextButtonView
import com.dropbox.base.analytics.AnalyticsLogger
import com.dropbox.base.analytics.ClientDeprecationEvents
import com.dropbox.base.build_configuration.DevBuild
import com.dropbox.core.client_deprecation.ForceNotDeprecatedInternalOnly
import com.dropbox.core.ui.components.buttons.DbxButtonFlatBlue

/**
 * Populates [ClientDeprecationUpdateActivityData] that will determine what elements on the UI to
 * show to the user.
 */
class ClientDeprecationUpdateActivity : BaseActivity() {

    fun getData(
        isInternalOrDebug: Boolean,
        emmHelper: EmmHelper,
        updateNagManager: UpdateNagManager,
        forceNotDeprecatedInternalOnly: ForceNotDeprecatedInternalOnly,
        analyticsLogger: AnalyticsLogger
    ):
            ClientDeprecationUpdateActivityData {
        lateinit var data: ClientDeprecationUpdateActivityData
        val title = R.string.client_deprecation_update_title

        when {
            isInternalOrDebug ->
                data = ClientDeprecationUpdateActivityData(
                        title = title,
                        subtitle = R.string.client_deprecation_update_internal_subtitle,
                        primaryButtomPair = Pair.create<Int, View.OnClickListener>(
                                R.string.client_deprecation_update_button, View.OnClickListener {
                                    val instructionsIntent = Intent(Intent.ACTION_VIEW)
                                    instructionsIntent.data =
                                            Uri.parse(AlphaBuildUpdateActivity.INSTRUCTIONS_URL)
                                    startActivity(instructionsIntent)
                                    ClientDeprecationEvents.Event().setType(
                                            ClientDeprecationEvents.ActionType.UPGRADE_CLICKED)
                                            .logTo(analyticsLogger)
                                }),
                        secondaryButtonPair = Pair.create<Int, View.OnClickListener>(
                                R.string.client_deprecation_update_debug_action,
                                View.OnClickListener {
                                    forceNotDeprecatedInternalOnly.value = true
                                    finish()
                                    activity.startActivity(
                                            Intent(activity, DropboxBrowser::class.java))
                        }))
            emmHelper.isInEmmMode ->
                data = ClientDeprecationUpdateActivityData(
                        title = title,
                        subtitle = R.string.client_deprecation_update_emm_subtitle,
                        primaryButtomPair = Pair.create<Int, View.OnClickListener>(
                                R.string.client_deprecation_update_button, View.OnClickListener {
                            updateNagManager.updateNow(this)
                            ClientDeprecationEvents.Event().setType(
                                    ClientDeprecationEvents.ActionType.UPGRADE_CLICKED)
                                    .logTo(analyticsLogger)
                        }))
            else ->
                data = ClientDeprecationUpdateActivityData(
                        title = title,
                        subtitle = R.string.client_deprecation_update_play_store_subtitle,
                        primaryButtomPair = Pair.create<Int, View.OnClickListener>(
                                R.string.client_deprecation_update_button, View.OnClickListener {
                                updateNagManager.updateNow(this)
                                ClientDeprecationEvents.Event().setType(
                                        ClientDeprecationEvents.ActionType.UPGRADE_CLICKED)
                                        .logTo(analyticsLogger)
                        }))
        }
        return data
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val view = FullscreenImageTitleTextButtonView(this)
        view.setImageResource(R.drawable.upgrade_folder)
        val emmHelper: EmmHelper = DropboxApplication.getEmmHelper(this)
        val updateNagManager: UpdateNagManager = DropboxApplication.getUpdateNagManager(this)
        val isForceNotDeprecatedInternalOnly: ForceNotDeprecatedInternalOnly = DropboxApplication
                .getForceNotDeprecatedInternalOnly(this)
        val analyticsLogger = DropboxApplication.getDefaultEventLogger(this)

        val data = getData(InternalBuild.IS_INTERNAL_BUILD || DevBuild.IS_DEV_BUILD, emmHelper,
                updateNagManager, isForceNotDeprecatedInternalOnly, analyticsLogger)
        val title = data.title
        val subtitle = data.subtitle
        val primaryButtonPair = data.primaryButtomPair
        val secondaryButtonPair = data.secondaryButtonPair

        view.setTitleText(title)
        view.setBodyText(subtitle)

        view.setButtonText(primaryButtonPair.first)
        view.setButtonOnClickListener(primaryButtonPair.second)

        if (secondaryButtonPair != null) {
            val secondaryButtom = DbxButtonFlatBlue(this)
            secondaryButtom.setText(secondaryButtonPair.first)
            secondaryButtom.setOnClickListener(secondaryButtonPair.second)
            view.setBottomContent(secondaryButtom)
        }
        setContentView(view)
    }

    override fun showsUpgradeNags(): Boolean {
        return false
    }

    override fun showsLockScreen(): Boolean {
        // Nothing sensitive here; don't need to unlock before showing this
        return false
    }

    override fun onBackPressed() {
        super.onBackPressed()
        val analyticsLogger = DropboxApplication.getDefaultEventLogger(this)
        ClientDeprecationEvents.Event().setType(
                ClientDeprecationEvents.ActionType.APP_BACKGROUNDED).logTo(analyticsLogger)
    }
}
