/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("ContentUriUtils")
package com.dropbox.android.util

import android.content.ContentResolver
import android.database.Cursor
import android.net.Uri
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.base.util.anonymize

private const val TAG = "ContentResolverUtils"

/**
 * Queries the content provider and fetch the string-valued column specified.
 * This will likely fail if you try to fetch non-string-valued columns using this method.
 */
@Throws(SecurityException::class)
internal fun ContentResolver.queryString(
    uri: Uri,
    columnName: String,
    selection: String? = null,
    selectionArgs: Array<String>? = null
): String? {
    var cursor: Cursor? = null
    try {
        cursor = this.query(uri, null, selection, selectionArgs, null)
        if (cursor != null && cursor.moveToFirst()) {
            val nameIndex = cursor.getColumnIndex(columnName)
            if (nameIndex < 0) {
                // The requested column doesn't exist, which is sometimes expected.
                DbxLog.external(
                        TAG,
                        "Couldn't get $columnName from URI: ${uri.anonymize()}")
                return null
            }
            val colType = cursor.getType(nameIndex)
            if (Cursor.FIELD_TYPE_STRING == colType) {
                return cursor.getString(nameIndex)
            }
        }
    } catch (e: SecurityException) {
        // We don't have permission to access things like Contacts.
        DbxLog.external(
                TAG,
                "Security exception, couldn't get $columnName from URI: ${uri.anonymize()}",
                e)
        throw e
    } catch (e: RuntimeException) {
        // Some URIs seem to die in the query.
        DbxLog.external(
                TAG, "Couldn't get $columnName from URI: ${uri.anonymize()}")
        throw e
    } finally {
        cursor?.close()
    }
    return null
}

/**
 * Safe version of [ContentProvider#getString()] that returns null.
 */
internal fun ContentResolver.queryStringOrNull(uri: Uri, columnName: String): String? {
    return try {
        queryString(uri, columnName)
    } catch (e: SecurityException) {
        null
    } catch (e: RuntimeException) {
        null
    }
}

/**
 * @return Given a Uri it tries to resolve the data column as a [String]. It swallows
 * [IllegalStateException] thrown when URI cannot be resolved to a Content Provider
 *
 * @throws [SecurityException] if we cannot query Content Provider
 */
@Throws(SecurityException::class)
fun ContentResolver.queryDataColumn(
    uri: Uri,
    selection: String? = null,
    selectionArgs: Array<String>? = null
): String? {
    return try {
        queryString(uri, "_data", selection, selectionArgs)
    } catch (e: RuntimeException) {
        // We saw an issue where ContentResolver is unable to find a provider for given uri
        // because we are getting an unknown URI "content://media/external". So for now
        // ignoring the exception as we can't do much in that case.
        // See: https://tails.corp.dropbox.com/T64439
        null
    }
}
