// dbx-min-coverage: 61% because grandfathered
/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("UriUtils")
package com.dropbox.android.util

import android.content.ContentResolver
import android.content.ContentUris
import android.content.Context
import android.net.Uri
import android.os.Environment
import android.provider.DocumentsContract
import android.provider.MediaStore
import android.provider.MediaStore.MediaColumns
import android.provider.OpenableColumns
import com.dropbox.android.localfile.DropboxFileCacheManager
import com.dropbox.base.oxygen.DbxLog
import com.dropbox.base.util.splitExt
import com.dropbox.base.util.stripIllegalDropboxFileChars
import com.dropbox.core.util.MimeUtils
import com.dropbox.base.util.anonymize
import com.dropbox.hairball.device_storage.UploadSourceSafetyChecker
import com.dropbox.product.dbapp.path.DropboxPath
import java.io.File
import java.util.HashMap
import java.util.Locale

private const val TAG = "UriUtils"

fun Uri.isDropboxUrl() = (this.isDropboxHttpUrl() || this.isDropboxHttpsUrl())

private fun Uri.isDropboxHttpUrl() = this.isDropboxUrl("http")

fun Uri.isDropboxHttpsUrl() = this.isDropboxUrl("https")

private fun Uri.isDropboxUrl(scheme: String): Boolean {
    if (this.host == null) {
        return false
    }

    val hostParts = this.host.toLowerCase(Locale.US).split('.').takeLast(2)

    return (scheme.equals(this.scheme, true) &&
            hostParts.size == 2 &&
            hostParts[0].equals("dropbox", true) &&
            hostParts[1].equals("com", true))
}

/**
 * @return True if the given uri lives on the paper.dropbox.com domain
 */
fun Uri.isPaperUrl(): Boolean {
    if (!isDropboxUrl()) {
        return false
    }

    val hostParts = this.host.toLowerCase(Locale.US).split('.').takeLast(3)

    // Check to see if this is a Paper URL.
    if (hostParts.size == 3 && hostParts.first().equals("paper", true)) {
        return true
    }

    // Check to see if this is a Dropbox URL redirecting to Paper.
    return this.query?.contains("paper.dropbox.com") ?: false
}

/**
 * Parse a string that either represents a Uri or a file path into a Uri object
 */
fun uriFromUriString(uri: String): Uri {
    return if (uri.startsWith("/")) {
        // handle raw file paths
        Uri.fromFile(File(uri))
    } else {
        Uri.parse(uri)
    }
}

/**
 * Try to get the filename from the URI's content provider.
 *
 * We query for both the _display_name and _data columns separately, and try to make a
 * pseudo-intelligent decision based on the results.
 *
 * Some content providers (e.g. contacts) don't have a _data column, so asking explicitly for both
 * together would throw an exception. Other providers don't have a _display_name column.
 *
 * And some providers (like QuickOffice in Android 4.4) throw an exception if you ask for all
 * columns using the null projection.
 */
private fun fileNameFromContentUri(contentResolver: ContentResolver, uri: Uri): String? {

    // Try to get file name using the _display_name column
    val displayFileName = contentResolver.queryStringOrNull(uri, OpenableColumns.DISPLAY_NAME)

    // Get the file name using the _data column
    var dataFileName: String? = null
    val dataFilePath = contentResolver.queryStringOrNull(uri, MediaColumns.DATA)
    if (dataFilePath != null) {
        dataFileName = File(dataFilePath).name
    }

    /*
     * Default to using the name of the local file if we either didn't get a display name, or if the
     * local file name starts with the display name but is a longer string, which means that the
     * display name is likely missing the file extension. This is a bug that popped up on some
     * HTC devices.
     */
    return if (displayFileName != null &&
            dataFileName != null &&
            dataFileName.startsWith(displayFileName)) {
        dataFileName
    } else {
        // displayFileName is preferred over dataFileName if both are non-null
        // but displayFileName is not a prefix of dataFileName
        displayFileName ?: dataFileName
    }
}

/**
 * Returns the name of the file at the given URI. If a file cannot be found at that location, or a
 * [SecurityException] occurs because we do not have access to the file, a best guess is returned -
 * either the last path segment of the URI or, failing that, the string "unknown".
 *
 * This function doesn't ensure that this file isn't a private Dropbox file.
 * Before viewing, uploading, or exposing this file that check should be
 * performed with [.getSafeFileForUploadFromUri]
 */
fun pickFileNameForUri(contentResolver: ContentResolver, uri: Uri): String {
    var fileName: String? = null
    val scheme = uri.scheme
    if (DropboxPath.isValidUri(uri)) {
        // We know how Dropbox uri's work
        fileName = DropboxPath(uri).name
    } else if (scheme == null) {
        // Perhaps it's a file.  This seems to happen sometimes.
        fileName = File(uri.toString()).name
    } else if (scheme == "file") {
        // "file://path"
        fileName = File(uri.path).name
    } else if (scheme == "content") {
        // Try to read the filename from the content provider
        fileName = fileNameFromContentUri(contentResolver, uri)
        if (fileName != null) {
            val nameExtPair = fileName.splitExt()
            // Handle the case for google drive if file does not has any ending. If it has
            // an ending just leave it as is. This needs to be handled specially for drive
            // because docs and spreadsheets are exported with mimetype as pdf but name does
            // not has any extension.
            if ("com.google.android.apps.docs.storage" == uri.authority &&
                    nameExtPair.second.isEmpty()) {
                val mimeType = contentResolver.queryString(uri, MediaColumns.MIME_TYPE)
                val extensionToAppend = MimeUtils.endingFromMimeType(mimeType)
                if (extensionToAppend != null) {
                    fileName += ".$extensionToAppend"
                }
            }
        }
    }

    if (fileName.isNullOrEmpty()) {
        val uriPath = uri.path
        // Check to see if it simply ends with the file hierarchy
        if (uriPath != null) {
            val possibleFile = File(uriPath)
            if (possibleFile.exists()) {
                fileName = possibleFile.name
            }
        }
    }

    // Fall back to URI path segment
    if (fileName.isNullOrEmpty()) {
        fileName = uri.lastPathSegment
    }

    if (fileName.isNullOrEmpty()) {
        fileName = "unknown"
    }

    // Remove disallowed characters like \n from the filename
    return fileName!!.stripIllegalDropboxFileChars()
}

/**
 * Gets a file from a content or file URI, or returns null if an associated
 * file cannot be found.
 *
 * @throws SecurityException if the file is in the app's private directory
 * or if we don't have permission to read from the URI.
 */
fun getSafeFileForUploadFromUri(
    contentResolver: ContentResolver,
    fileCacheMgr: DropboxFileCacheManager,
    uploadSourceSafetyChecker: UploadSourceSafetyChecker,
    uri: Uri
): File? {
    val filename: String?

    val scheme = uri.scheme
    if (scheme == null) {
        // Perhaps it's a file.  This seems to happen sometimes.
        filename = uri.toString()
    } else if (scheme == "file") {
        filename = uri.path
    } else if (DropboxPath.isValidUri(uri)) {
        // We know how Dropbox uri's work
        val path = DropboxPath(uri)
        return fileCacheMgr.getFile(path).toFile()
    } else {
        // Check to see if it simply ends with the file hierarchy
        val possibleFileName = uri.path
        val possibleFile = File(possibleFileName)
        if (possibleFile.exists()) {
            return uploadSourceSafetyChecker.returnFileOnlyIfSafeForUpload(possibleFile)
        } else {
            filename = contentResolver.queryDataColumn(uri)
        }
    }

    if (filename == null) {
        DbxLog.external(
                TAG,
                "Unable to determine source File from uri: ${uri.anonymize()}")
    } else {
        val file = File(filename)
        if (file.exists()) {
            return uploadSourceSafetyChecker.returnFileOnlyIfSafeForUpload(file)
        } else {
            DbxLog.external(
                    TAG, "File doesn't exist: ${file.anonymize()}}")
        }
    }
    return null
}

fun isPrivateUri(
    contentResolver: ContentResolver,
    fileCacheMgr: DropboxFileCacheManager,
    uploadSourceSafetyChecker: UploadSourceSafetyChecker,
    uri: Uri
): Boolean {
    val scheme = uri.scheme
    if (scheme == null || "file" == scheme) {
        return try {
            getSafeFileForUploadFromUri(
                    contentResolver, fileCacheMgr, uploadSourceSafetyChecker, uri) == null
        } catch (e: SecurityException) {
            true
        }
    }

    // if it's not a file uri, then it's not private
    return false
}

/**
 * @return A map of file name to uri. Uses the file path in the collection and
 * ContentResolver to determine the name of the file using [.pickFileNameForUri]. If this
 * function returns same name of the uri, like in the case of Google Photos app it returns
 * "image" for all, we add an integer id to the returned name to distinguish them.
 */
fun getCanonFileNamesFromPaths(
    cr: ContentResolver,
    filePaths: Collection<Uri>
): HashMap<String, Uri> {
    val fileNames = HashMap<String, Uri>(filePaths.size)
    var i = 1
    for (uri in filePaths) {
        // Certain providers like google photos always provide "image" as the name for all
        // photos
        var name = pickFileNameForUri(cr, uri).toLowerCase(Locale.US)
        if (fileNames.containsKey(name)) {
            name += i
            i++
        }
        fileNames[name] = uri
    }
    return fileNames
}

/**
 * Get the file path for a uri. This is a convoluted way to get the path for an Uri created using
 * the StorageAccessFramework. This in no way is the official way to do this but there does not seem
 * to be a better way to do this at this point. It is taken from
 * https://github.com/iPaulPro/aFileChooser.
 *
 * @param context The context of the application
 * @param uri The uri of the saved file
 * @return The file with path pointing to the saved file. It can return null if we can't resolve the uri properly.
 */
fun Uri.getFileForUri(context: Context): File? {
    var path: String? = null
    if (DocumentsContract.isDocumentUri(context, this)) {
        val docId = DocumentsContract.getDocumentId(this)

        // ExternalStorageProvider
        if (isExternalStorageDocument()) {
            val split = docId.split(":")
            val type = split[0]

            if ("primary".equals(type, ignoreCase = true)) {
                path = Environment.getExternalStorageDirectory().toString() + "/" + split[1]
            }
        } else if (isDownloadsDocument()) {
            // DownloadsProvider
            path = if (docId.startsWith("raw:")) {
                docId.replaceFirst("raw:".toRegex(), "")
            } else {
                val contentUri = ContentUris.withAppendedId(
                        Uri.parse("content://downloads/public_downloads"),
                        java.lang.Long.valueOf(docId))

                if (contentUri != null) {
                    context.contentResolver.queryDataColumn(contentUri)
                } else {
                    return null
                }
            }
        } else if (isMediaDocument()) {
            // MediaProvider
            val split = docId.split(":")
            val type = split[0]

            val contentUri: Uri? = when (type) {
                "image" -> MediaStore.Images.Media.EXTERNAL_CONTENT_URI
                "video" -> MediaStore.Video.Media.EXTERNAL_CONTENT_URI
                "audio" -> MediaStore.Audio.Media.EXTERNAL_CONTENT_URI
                else -> null
            }

            val selection = "_id=?"
            val selectionArgs = arrayOf(split[1])

            path = if (contentUri != null) {
                context.contentResolver.queryDataColumn(contentUri, selection, selectionArgs)
            } else {
                return null
            }
        }
    } else {
        path = getFilePathFromScheme(context, this)
    }

    return if (path != null) {
        File(path)
    } else null
}

private fun getFilePathFromScheme(context: Context, uri: Uri): String? {
    val scheme = uri.scheme ?: return null

    return when (scheme.toLowerCase()) {
        // MediaStore (and general)
        "content" -> context.contentResolver.queryDataColumn(uri)

        // File
        "file" -> uri.path

        else -> null
    }
}

private fun Uri.isExternalStorageDocument() = "com.android.externalstorage.documents" == authority

private fun Uri.isDownloadsDocument() = "com.android.providers.downloads.documents" == authority

private fun Uri.isMediaDocument() = "com.android.providers.media.documents" == authority
