/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("LocalPathResolver")

package com.dropbox.hairball.device_storage

import com.dropbox.base.filesystem.DbxFileUtils
import com.dropbox.base.util.splitExt
import com.dropbox.product.dbapp.path.DropboxPath
import com.dropbox.product.dbapp.path.ExternalPath
import com.dropbox.product.dbapp.path.Path
import com.dropbox.product.dbapp.path.PathVisitor
import com.dropbox.product.dbapp.path.SharedLinkPath
import com.google.common.hash.Hashing
import org.apache.commons.io.Charsets
import java.io.File

/**
 * Utility functions for mappings paths to the their corresponding files/paths
 * on the local filesystem.
 */

/**
 * Returns the external file that this path points to for this external storage.
 */
fun DropboxPath.getLocalFile(externalStorage: ExternalStorage): File {
    return File(externalStorage.fileCacheRoot, asFilePath())
}

/**
 * @return The root local storage directory for this path.
 */
fun DropboxPath.getLocalFileRoot(externalStorage: ExternalStorage): File {
    return externalStorage.fileCacheRoot
}

/**
 * Returns the local file that this path points to for this local storage.
 *
 * NOTE: if the type of [Path] is unknown, call [com.dropbox.android.util.PathHelper.getLocalFile]
 */
fun SharedLinkPath.getLocalFile(externalStorage: ExternalStorage): File {
    // Retain the original file extension because some external apps use it when opening the file.
    val ext = name.splitExt().second
    val file = File(externalStorage.fileCacheRoot, md5Hex(urlPath) + ext)
    return if (relativePath.isPresent) {
        File(file, md5Hex(relativePath.get()) + ext)
    } else file
}

@Suppress("DEPRECATION")
private fun md5Hex(str: String): String {
    return Hashing.md5().hashString(str, Charsets.UTF_8).toString()
}

/**
 * @return The root local storage directory for this path.
 */
fun SharedLinkPath.getLocalFileRoot(externalStorage: ExternalStorage): File {
    return externalStorage.fileCacheRoot
}

/**
 * Returns the file on the filesystem corresponding to this path. This is where the copy of the
 * external file should live (if it's not been copied yet) or actually lives (if it's been
 * successfully copied).
 */
fun ExternalPath.getLocalFile(storage: ExternalLocalStorage): File {
    return File(storage.localDocumentPreviewCacheRoot, name)
}

/**
 * A string to use for uniquely identifying this [DropboxPath] that can be used as a file path.
 */
fun DropboxPath.asFilePath(): String {
    @Suppress("DEPRECATION")
    return DbxFileUtils.escapeForLocalFilePath(rawPath)
}

/**
 * A string to use for uniquely identifying this [SharedLinkPath] that can be used as a file path.
 */
fun SharedLinkPath.asFilePath() = DbxFileUtils.escapeForLocalFilePath(canonicalName)

/**
 * A string to use for uniquely identifying this [ExternalPath] that can be used as a file path.
 */
fun ExternalPath.asFilePath() = asCanonicalPath()

fun <T : Path> T.asFilePath(): String {
    return accept(object : PathVisitor<String> {
        override fun visit(path: DropboxPath?) = checkNotNull(path).asFilePath()
        override fun visit(path: SharedLinkPath?) = checkNotNull(path).asFilePath()
        override fun visit(path: ExternalPath?) = checkNotNull(path).asFilePath()
    })
}
