/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
package com.dropbox.hairball.entry

/**
 * Represents different classes of cloud docs, such as canonical representation and cloud-backed
 * files (regular files with cloud capabilities).
 */
enum class CloudDocClass constructor(val intValue: Int) {
    //
    // Note: Do not change the int value used to represent existing values of this enum because
    // they may be used to serialise this value on disk.
    // Only add new values with distinct int representations.
    //
    // Note: Dealing with the SDK requires us to treat unknown values as other before we serialise
    // the value. that means even if the client is updated to understand new types of cloud docs,
    // until the value gets resent from the server and re-serialised, it will still be stored
    // as {@link #OTHER}.
    // Therefore, if you are adding a new type of cloud doc, you may need to do a no-op migration
    // of the relevant table to flush all existing OTHER values correctly.
    //
    /**
     * This is a regular file or folder with no cloud properties.
     */
    NONE(0),
    /**
     * This is some future type of cloud doc whose value that we couldn't understand (either
     * in the JSON or via the SDK).
     */
    OTHER(1),
    /**
     * This is a cloud doc existing outside the filesystem e.g. a Paper doc or a Google Sheet
     */
    CANONICAL(2),
    /**
     * This is a regular file that has some cloud properties i.e. a Microsoft Word file with
     * collaborative  editor available.
     */
    CLOUD_BACKED_FILE(3);

    companion object {
        /**
         * Convert from string representation such as provided via JSON to {@link CloudDocClass}.
         * Returns {@link #OTHER} if the value is not understood.
         */
        @JvmStatic
        fun cloudDocClassFromJsonString(jsonString: String): CloudDocClass {
            val string = jsonString.toLowerCase()
            return when (string) {
                "canonical" -> CANONICAL
                "cloud_backed_file" -> CLOUD_BACKED_FILE
                else -> OTHER
            }
        }

        /**
         * For deserializing from stored value.
         * @throws IllegalArgumentException if int value does not have a corresponding class
         */
        @JvmStatic
        fun fromInt(intValue: Int): CloudDocClass {
            return CloudDocClass.values().associateBy { it.intValue }[intValue]
                    ?: throw IllegalArgumentException(
                        "Value with no corresponding CloudDocClass: " + intValue)
        }
    }
}
