/*
 * Copyright (c) 2018, Dropbox, Inc. All rights reserved.
 */
@file:JvmName("PathUtils")

package com.dropbox.hairball.path

import com.dropbox.base.filesystem.DbxFileUtils
import com.dropbox.hairball.device_storage.UserExternalStorage
import com.dropbox.hairball.entry.SharedLinkEntry
import com.dropbox.product.dbapp.path.DropboxPath
import com.dropbox.product.dbapp.path.SharedLinkPath
import java.io.File

/**
 * Given a local file and a local storage, try to get the DropboxPath that it refers to.
 *
 * @throws IllegalArgumentException if the file is not a cached Dropbox file.
 */
fun parseFromFile(userExternalStorage: UserExternalStorage, file: File): DropboxPath {
    if (!userExternalStorage.checkFilePathInLocalDropbox(file, true)) {
        throw IllegalArgumentException("Not a valid local path: $file")
    }

    val root = userExternalStorage.fileCacheRoot.path
    var filePath = file.path
    if (filePath == root) {
        return DropboxPath.ROOT
    }

    filePath = filePath.replaceFirst(root.toRegex(), "")
    return DropboxPath(DbxFileUtils.unescapeFromLocalFilePath(filePath), file.isDirectory)
}

/**
 * Creates the shared link path we should store from a request path and a remote entry.
 * The result path will have the same url path as the request, but its relative path will
 * either be the path from the remote entry or null.
 */
fun fromRemoteEntry(
    requestPath: SharedLinkPath,
    remoteEntry: SharedLinkEntry,
    isPartOfDirectoryContents: Boolean
): SharedLinkPath {
    var remotePath = remoteEntry.path
    val keepRemotePath = requestPath.relativePath.isPresent || isPartOfDirectoryContents
    if (!keepRemotePath) {
        remotePath = null
    }

    val fileName: String?
    fileName = if (remotePath == null) {
        requestPath.name
    } else {
        null
    }

    return SharedLinkPath(requestPath.urlPath, fileName, remotePath, remoteEntry.isDir)
}
